

# _*_ coding: utf-8 _*_

'''
画雷达综合图主程序

'''


from metradar.io.decode_fmt_pyart import read_cnrad_fmt
import pyart
import os
import numpy as np
import numpy.ma as MA
from metradar.io.decode_pup_rose import READ_ROSE
import matplotlib.pyplot as plt
import cartopy.crs as ccrs

from netCDF4 import num2date
import xarray 
from cartopy.io.shapereader import Reader
from matplotlib.font_manager import FontProperties
import pandas as pd 
from metpy.calc import wind_components
from metpy.units import units
from metradar.util.parse_pal import parse_pro
from matplotlib.colorbar import ColorbarBase
import matplotlib.patheffects as path_effects
from matplotlib import patheffects
import matplotlib.transforms as transforms 
from matplotlib.transforms import offset_copy
from metradar.core.oa_dig_func import Object_Analyst
import warnings
warnings.filterwarnings('ignore')
import matplotlib as mpl

from metradar.core.get_cross_section import get_cross_radar
import matplotlib
# matplotlib.use('Agg')

from matplotlib import ticker
import xarray as xr
from metradar.config import CONFIG

# 资源文件路径
RESOURCES_PATH = CONFIG.get('SETTING','RESOURCES_PATH')
MAP_PATH = RESOURCES_PATH + '/maps'


# 初始化参数字典，用于存储绘图所需的所有参数
def ini_params():
    param={'radarfile_path':None, # 雷达文件路径
            'radarfile_name':None, # 雷达文件名 
            'rose_cr_path':None, # ROSE组合反射率因子文件路径
            'rose_cr_name':None, # ROSE组合反射率因子文件名
            'mosaicfile_path':None,# 反射率拼图文件路径
            'mosaicfile_name':None, # 反射率拼图文件名
            'pic_path':None, # 图片保存路径
            'timestr':None, # 图片时间
            'aws_min_file_path':None, # aws分钟数据文件路径
            'aws_min_file_name':None, # aws分钟数据文件名
            'aws_hour_file_path':None, # aws小时数据文件路径
            'aws_hour_file_name':None,# aws小时数据文件名
            'rose_cr_path':None, # ROSE组合反射率因子文件路径
            'rose_cr_name':None, # ROSE组合反射率因子文件名
            'aws_min_delta_t_file_path':None, # aws小时变温数据文件路径
            'aws_min_delta_t_file_name':None,# aws小时变温数据文件名
            'aws_min_delta_p_file_path':None, # aws小时变压数据文件路径
            'aws_min_delta_p_file_name':None,# aws小时变压数据文件名
            'wind_bar_width_inner':None,#wind bar 里线宽
            'wind_bar_width_outter':None,#wind bar 外线宽
            'gis_name':None, # 站点名称
            'gis_lats':None, # 站点纬度
            'gis_lons':None, # 站点经度
            'slat':None, # 绘图区域南纬
            'nlat':None, # 绘图区域北纬
            'wlon':None, # 绘图区域西经
            'elon':None, # 绘图区域东经
            'ref_colorfile':None, # 回波强度色标文件
            'vel_colorfile':None, # 径向速度色标文件
            'cc_colorfile':None, # 相关系数色标文件
            'zdr_colorfile':None, # 差分反射率色标文件
            'kdp_colorfile':None, # 差分相移率色标文件
            'fontfile':None, # 字体文件
            'dpi':300, # 图片分辨率
            'pic_format':'png', # 图片格式
            'figsize_width':4, # 图片宽度
            'figsize_height':4, # 图片高度
            'fontsize_gis':4, # GIS字体大小
            'fontsize_colorbar':3, # 色标字体大小
            'fontsize_title':4,# 标题字体大小
            'fontsize_tick':6, # 坐标轴刻度字体大小
            'bdraw_title_ppi':True, # 是否给PPI图绘制标题
            'bdraw_title_crs':True, # 是否给垂直剖面图绘制标题
            'mapcolor':[170/255,170/255,170/255], # 行政边界颜色
            'linewidth_map': 0.5,# 行政边界线宽
            'contour_color':[0/255,0/255,0/255], # 等值线颜色
            'linewidth_cntr': 0.8,# 等值线线宽
            'breplace':True, #如果图片文件已存在，是否重新绘制
            'bdraw_crs':False, # 是否绘制垂直剖面图
            'thred_pre1h': 30, # 1小时降水阈值
            'thred_ref':0, # 回波强度阈值
            'radarname': 'XX',# 雷达站名
            
        }
    return param


# FuncFormatter can be used as a decorator
@ticker.FuncFormatter
def major_formatter_lon(x, pos):
    return f'{x:.1f}°E'

@ticker.FuncFormatter
def major_formatter_lat(x, pos):
    return f'{x:.1f}°N'

class DRAW_RADAR_OTHER:
    
    def __init__(self,params):
        pass
        self.radarfile_path=params['radarfile_path']
        self.radarfile_name=params['radarfile_name']
        self.rose_cr_path=params['rose_cr_path']
        self.rose_cr_name=params['rose_cr_name']
        self.mosaicfile_path=params['mosaicfile_path']
        self.mosaicfile_name=params['mosaicfile_name']
        self.aws_min_file_path=params['aws_min_file_path']
        self.aws_min_file_name=params['aws_min_file_name']

        self.aws_min_delta_t_file_path=params['aws_min_delta_t_file_path']
        self.aws_min_delta_t_file_name=params['aws_min_delta_t_file_name']
        self.aws_min_delta_p_file_path=params['aws_min_delta_p_file_path']
        self.aws_min_delta_p_file_name=params['aws_min_delta_p_file_name']

        self.aws_hour_file_path=params['aws_hour_file_path']
        self.aws_hour_file_name=params['aws_hour_file_name']
        self.gis_name=params['gis_name']
        self.gis_lats=params['gis_lats']
        self.gis_lons=params['gis_lons']
        self.slat=params['slat']
        self.nlat=params['nlat']
        self.wlon=params['wlon']
        self.elon=params['elon']
        self.wind_bar_width_inner=params['wind_bar_width_inner']
        self.wind_bar_width_outter=params['wind_bar_width_outter']
        self.ref_colorfile=params['ref_colorfile']
        self.vel_colorfile=params['vel_colorfile']
        self.cc_colorfile=params['cc_colorfile']
        self.zdr_colorfile=params['zdr_colorfile']
        self.fontfile=params['fontfile']
        self.picpath=params['pic_path']
        self.timestr=params['timestr']
        self.breplace=params['breplace']
        self.mapcolor=params['mapcolor']
        self.linewidth_map=params['linewidth_map']
        self.contour_color=params['contour_color']
        self.linewidth_cntr=params['linewidth_cntr']
        self.dpi = params['dpi']
        self.paintsize_x = params['figsize_width']
        self.paintsize_y = params['figsize_height']

        self.gatefilter_ref = None
        self.pic_format = params['pic_format']
        self.fontsize_gis=params['fontsize_gis']
        self.fontsize_colorbar=params['fontsize_colorbar']
        self.fontsize_title=params['fontsize_title']
        self.bdraw_title_ppi=params['bdraw_title_ppi']
        self.bdraw_title_crs=params['bdraw_title_crs']
        self.crs_paintsize_x = params['figsize_width']
        self.bdraw_crs=params['bdraw_crs']
        self.fontsize_tick=params['fontsize_tick']
        self.thred_pre1h=params['thred_pre1h']
        self.thred_ref =params['thred_ref']
        self.radarname = params['radarname']
        self.linewidth_cntr = params['linewidth_cntr']
        if self.bdraw_crs == True:
            self.crs_start_azi=params['crs_start_azi']
            self.crs_end_azi=params['crs_end_azi']
            self.crs_start_range=params['crs_start_range']
            self.crs_end_range=params['crs_end_range']
            self.top_height = params['top_height']
            
        else:
            self.crs_start_azi=None
            self.crs_end_azi=None
            self.crs_start_range=None
            self.crs_end_range=None
            self.top_height = None

        try:
            if not os.path.exists(self.picpath): os.makedirs(self.picpath)
        except:
            pass

        self.ratio = (self.nlat - self.slat)/(self.elon-self.wlon)

        
        # 设置坐标轴字体大小

        mpl.rcParams['font.size'] = self.fontsize_tick
        mpl.rcParams['font.weight'] = 'bold'
        # mpl.rcParams['font.family'] = 'Times New Roman'
        mpl.rcParams['xtick.direction'] = 'in'  # 'out'  'inout'
        mpl.rcParams['ytick.direction'] = 'in'  # 'out'  'inout'


        self.cref=None
        self.voltime = None
        
        self.disratio =  self.paintsize_x / 2
        # self.disratio=1

        self.font_gis=FontProperties(fname=self.fontfile, size=self.fontsize_gis)
        self.font_colorbar=FontProperties(fname=self.fontfile, size=self.fontsize_colorbar)
        self.font_indicate=FontProperties(fname=self.fontfile, size=self.fontsize_colorbar*2)
        self.font_title=FontProperties(fname=self.fontfile, size=self.fontsize_title)
        self.stroke_line_width=1 * self.disratio
        self.cref=None
        self.voltime = None
        self.crs_data=None
        self.crs_fig=None

        # self.axes_pos = [0.01, 0.06, 0.98, 0.88]
        self.axes_pos = [0.05, 0.07, 0.9, 0.9]
        self.cbaxes_pos = [0.05, 0.01, 0.9, 0.05]
        # self.cbaxes_pos = [0.8,0.15,0.02,0.6]

        self.crs_axes_pos = [0.15, 0.15, 0.70, 0.8]
        self.crs_cbaxes_pos = [0.86, 0.15, 0.03, 0.8]
        self.radar = None

    def read_vol_data(self,):
        # filepath = 'data/'
        # filename = 'ZZHN.20210720.180000.ar2v'
        if self.radarfile_name is None:
            print('no valid radfile!')
            return False
        if not os.path.exists(self.radarfile_path + os.sep + self.radarfile_name):
            print(self.radarfile_path + os.sep + self.radarfile_name + ' not exists!')
            return False
        self.radar = read_cnrad_fmt(self.radarfile_path + os.sep + self.radarfile_name)
        # self.radar.fields.keys()
        self.display = pyart.graph.RadarMapDisplay(self.radar)
        self.gatefilter_ref = pyart.correct.GateFilter(self.radar)
        self.gatefilter_ref.exclude_below('reflectivity', self.thred_ref, exclude_masked=True, op='or', inclusive=False)
    
        
        self.g_rad_lon = self.radar.longitude['data'][0]
        self.g_rad_lat = self.radar.latitude['data'][0]

        times_t = self.radar.time['data'][0]
        units_t = self.radar.time['units']
        calendar = self.radar.time['calendar']
        voltime= num2date(times_t, units_t, calendar, only_use_cftime_datetimes=False,
                        only_use_python_datetimes=True)
        datetime_format='%Y%m%d-%H:%M'
        self.voltime = voltime.strftime(datetime_format)
        return True

    def get_crs_from_radar(self,):
        pass
        param_crs=dict()
        param_crs['crs_start_azi'] = self.crs_start_azi
        param_crs['crs_start_range'] = self.crs_start_range
        param_crs['crs_end_azi'] = self.crs_end_azi
        param_crs['crs_end_range'] = self.crs_end_range
        param_crs['top_height'] = self.top_height
        # st = time.time()
        if self.radar is None:
            return False
   
        self.crs_data = get_cross_radar(self.radar,param_crs)
        pass

    def draw_crs(self,subdir='垂直剖面',varname=None,brefresh=False,thred=None,):
        # check 
    
        if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.jpg') and not brefresh:
            return False
        if self.crs_data is None:
            print('warning: crs_data not pre loaded!')
            return False

        if varname is None:
            print('warning: varname is None!')
            return False

        if not thred is None:
            flag = self.crs_data[varname] < thred
            self.crs_data[varname] = MA.masked_array(self.crs_data[varname], mask=flag)

        colorfile = self.ref_colorfile
        if varname.find('ref') >=0:
            colorfile = self.ref_colorfile
        elif varname.find('vel') >=0:
            colorfile = self.vel_colorfile
        else:
            print('warning: varname = %s not config the colorfile in the draw_crs function!'%varname)
            return False

        colordata = parse_pro(colorfile)

        self.crs_ratio = 0.6
        self.crs_paintsize_y = self.crs_paintsize_x * self.crs_ratio
        self.crs_fig = plt.figure(figsize=(self.crs_paintsize_x,self.crs_paintsize_y))
        map_panel_axes = self.crs_axes_pos
        self.crs_ax = self.crs_fig.add_axes(map_panel_axes)
        plt.pcolormesh(self.crs_data[varname],cmap=colordata['cmap'],norm=colordata['norm'])
        xticks = self.crs_ax.get_xticks()
        if xticks[-1] < self.crs_data[varname].shape[1]:
            xticks.append(self.crs_data[varname].shape[1])
        else:
            xticks[-1] = self.crs_data[varname].shape[1]

        xticklabels = []
        for ix in range(len(xticks)):
           xticklabels.append('%.1f'%(xticks[ix]*self.crs_data['xreso']))
        xticklabels[0]='A'
        xticklabels[-1]='B'
        
        self.crs_ax.set_xticks(xticks)
        self.crs_ax.set_xticklabels(xticklabels)

        yticks = self.crs_ax.get_yticks()
        if yticks[-1] < self.crs_data[varname].shape[0]:
            yticks.append(self.crs_data[varname].shape[0])
        yticklabels = []
        for iy in range(len(yticks)):
           yticklabels.append('%.1f'%(yticks[iy]*self.crs_data['yreso']))
        
        self.crs_ax.set_yticklabels(yticklabels)
        self.crs_ax.set_xlabel('水平距离(KM)',font=self.font_colorbar)
        self.crs_ax.set_ylabel('垂直高度(KM)',font=self.font_colorbar)
        plt.grid()
        self.draw_title_crs(varname=varname)
        self.draw_crs_colorbar(colorfile=colorfile)
        self.crs_finish_save(subdir=subdir,add_str=varname,)

    def draw_title_crs(self,varname='reflectivity'):
        if not self.bdraw_title_crs:
            return None
        x,y,z=pyart.core.antenna_to_cartesian(self.crs_start_range, self.crs_start_azi, elevations=0)
        start_lon,start_lat=pyart.core.cartesian_to_geographic_aeqd(x,y,self.g_rad_lon,self.g_rad_lat)
        x,y,z=pyart.core.antenna_to_cartesian(self.crs_end_range, self.crs_end_azi, elevations=0)
        end_lon,end_lat=pyart.core.cartesian_to_geographic_aeqd(x,y,self.g_rad_lon,self.g_rad_lat)
        self.crs_startlat = start_lat[0]
        self.crs_startlon = start_lon[0]
        self.crs_endlat = end_lat[0]
        self.crs_endlon = end_lon[0]
        titlestr = 'A(%.2f,%.2f)->B(%.2f,%.2f) '%(self.crs_startlon,self.crs_startlat,self.crs_endlon,self.crs_endlat)
        tstr=''
        if self.voltime is None:
            tstr = self.timestr
        else:
            tstr = self.voltime
        plt.title('%s'%(titlestr),fontproperties=self.font_title,loc='right',verticalalignment='top')

    def draw_crs_colorbar(self,fig=None,colorfile=None,tickstep=5,orientation = 'vertical',cb_ratio=0.618):
        # 画色标
        ax_pos = self.crs_ax.get_position().bounds
        cbar_axes = [ax_pos[0]+ ax_pos[2]+0.005,(ax_pos[1]+ax_pos[1]+ax_pos[3])/2-ax_pos[3]*cb_ratio/2,0.02,ax_pos[3]*cb_ratio]
        cb_ax = self.crs_fig.add_axes(cbar_axes)
        colordata = parse_pro(colorfile)
        
        cbar = ColorbarBase(cb_ax, orientation=orientation, norm=colordata['norm'], cmap=colordata['cmap'],extend='both')
        cbar.set_ticks([])
        # cb_ax.axis('tight')

        trans = transforms.blended_transform_factory(cb_ax.transData, cb_ax.transAxes)
        tick_poss=np.arange(colordata['norm'].vmin, colordata['norm'].vmax, tickstep)
        xrange = colordata['norm'].vmax - colordata['norm'].vmin

        for tick_pos in tick_poss:
            cb_ax.hlines(tick_pos, colordata['norm'].vmin, colordata['norm'].vmin + 0.3*xrange, color="black",linewidth=0.5)
            cb_ax.hlines(tick_pos, colordata['norm'].vmin + 0.7*xrange, colordata['norm'].vmax, color="black",linewidth=0.5)
  
  

            cb_ax.text(1.04,tick_pos, str(int(tick_pos)),va="center", ha="left",fontsize=5) # font=self.font_colorbar

        cb_ax.text(1.8, 1.035, colordata['units'], transform=cb_ax.transAxes,va="center", ha="center", fontsize=5 ) # font=self.font_colorbar 
                  
        

    def add_crs_line(self,ax=None):
        pass
        if not self.bdraw_crs:
            # print('warning: bdraw_crs is False!')
            return False

        if ax is None:
            print('warning: ax is None in function add_crs_line!')
            return False

        x,y,z=pyart.core.antenna_to_cartesian(self.crs_start_range, self.crs_start_azi, elevations=0)
        start_lon,start_lat=pyart.core.cartesian_to_geographic_aeqd(x,y,self.g_rad_lon,self.g_rad_lat)
        x,y,z=pyart.core.antenna_to_cartesian(self.crs_end_range, self.crs_end_azi, elevations=0)
        end_lon,end_lat=pyart.core.cartesian_to_geographic_aeqd(x,y,self.g_rad_lon,self.g_rad_lat)
        self.crs_startlat = start_lat[0]
        self.crs_startlon = start_lon[0]
        self.crs_endlat = end_lat[0]
        self.crs_endlon = end_lon[0]

        simble_color=[0,0,0]
        ax.plot([start_lon[0],end_lon[0]],[start_lat[0],end_lat[0]],color='k',linewidth=0.75)
        ax.plot([start_lon[0],end_lon[0]],[start_lat[0],end_lat[0]],color=simble_color,linewidth=0.5)
        ax.text(start_lon[0],start_lat[0],'A',color=simble_color,font=self.font_colorbar,
                path_effects=[path_effects.Stroke(linewidth=0.25, foreground='black'),path_effects.Normal()])
        ax.text(end_lon[0],end_lat[0],'B',color=simble_color,font=self.font_colorbar,
                path_effects=[path_effects.Stroke(linewidth=0.25, foreground='black'),path_effects.Normal()])

    # 从拼图文件中读取组合反射率
    def get_cref_from_mosaicfile(self,):
        
        data = None
        if not os.path.exists(self.mosaicfile_path + os.sep + self.mosaicfile_name):
            print(self.mosaicfile_path + os.sep + self.mosaicfile_name + ' does not exists!')
            return False
        try:
            data = xarray.open_dataset(self.mosaicfile_path + os.sep + self.mosaicfile_name)
        except:
            print(self.mosaicfile_path + os.sep + self.mosaicfile_name + ' read error!')
            self.cref=None
            return False
        cref = data.cref.data
        self.cref = (cref.astype(np.float32) - data.cref.offset)/data.cref.scale
        
        self.cref = self.cref.transpose()
        self.cref = MA.masked_array(self.cref, mask=self.cref==-32)
        self.grid_lat,self.grid_lon=np.meshgrid(data.lat.data,data.lon.data)

        return True

    # 从ROSE产品中读取组合反射率因子数据
    def get_cref_from_rose(self,):
        
        rose_reader = READ_ROSE()
        data = None
        if not os.path.exists(self.rose_cr_path + os.sep + self.rose_cr_name):
            print(self.rose_cr_path + os.sep + self.rose_cr_name + ' does not exists!')
            return False
        try:
            data = rose_reader.read_cr(self.rose_cr_path, self.rose_cr_name)
        except:
            print(self.rose_cr_path + os.sep + self.rose_cr_name + ' read error!')
            self.cref=None
            return False
        cref = data.data
        
        self.cref = cref.transpose()
        self.cref = MA.masked_array(self.cref, mask=self.cref==-32)
        self.grid_lat,self.grid_lon=np.meshgrid(data.lat.data,data.lon.data)

        return True
    
    # 从单站雷达基数据中获取组合反射率因子，先进行三维插值，再求CR
    def get_cref_from_radar(self,xlimits=[-50,50], ylimits=[-50,50], zlimits=[1,15], xreso=1000, yreso=1000, zreso=500, grid_origin=None):
        print('enter get_cref_from_radar function!')
        
        self.xlims=xlimits # km
        self.ylims=ylimits # km
        self.zlims=zlimits # km
        self.xreso=xreso #m
        self.yreso=yreso #m
        self.zreso=zreso #m
        self.g_xlim = (self.xlims[0]*1000,self.xlims[1]*1000)
        self.g_ylim = (self.ylims[0]*1000,self.ylims[1]*1000)
        self.g_zlim = (self.zlims[0]*1000,self.zlims[1]*1000)
        self.g_numx = int(len(range(self.xlims[0],self.xlims[1]))*1000/self.xreso+1) # x方向格点数
        self.g_numy = int(len(range(self.ylims[0],self.ylims[1]))*1000/self.yreso+1) # y方向格点数
        self.g_numz = int(len(range(self.zlims[0],self.zlims[1]))*1000/self.zreso+1) # z方向格点数
        if grid_origin is None:
            grid_origin = (self.radar.latitude['data'][0], self.radar.longitude['data'][0])
        print('grid shape:',(self.g_numz, self.g_numy, self.g_numx))
        print('grid limits:',(self.g_zlim, self.g_ylim, self.g_xlim))
        print('grid origin:',grid_origin)
        self.grid=pyart.map.grid_from_radars(
            (self.radar,),
            grid_origin=grid_origin,
            weighting_function = 'BARNES2',
            grid_shape=(self.g_numz, self.g_numy, self.g_numx),
            grid_limits=(self.g_zlim, self.g_ylim, self.g_xlim),
            fields=['reflectivity'])

        xrdata = self.grid.to_xarray()

        self.cref = np.nanmax(xrdata.reflectivity.data[0],axis=0) 
        self.cref = self.cref.transpose()
        self.grid_lat = xrdata.lat.data
        self.grid_lon = xrdata.lon.data
        

    def draw_cref(self,):
        cmapdic= parse_pro(self.ref_colorfile)
        plt.pcolormesh(self.grid_lon,self.grid_lat,self.cref,cmap=cmapdic['cmap'],norm=cmapdic['norm'])  
    
    def draw_divergence(self,zorder=2):
        # cmapdic= parse_pro(self.ref_colorfile)
        # plt.pcolormesh(self.grid_lon,self.grid_lat,self.cref,cmap=cmapdic['cmap'],norm=cmapdic['norm'])  
        # 叠加涡度场
        # filepath_minute='temp/backup_aws_minute'
        # filename_aws='surface_aws_20210720_1800.csv'
        if self.aws_min_file_name is None:
            print('warning: aws_min_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_file_path + os.sep + self.aws_min_file_name):
            print(self.aws_min_file_path + os.sep + self.aws_min_file_name + ' does not exists!')
            return False
        awsdata = pd.read_csv(self.aws_min_file_path + os.sep + self.aws_min_file_name,encoding='GBK')

        aws_lats=awsdata['Lat'].values
        aws_lons=awsdata['Lon'].values
        aws_2minspd=awsdata['WIN_S_Avg_2mi'].values
        aws_2mindir=awsdata['WIN_D_Avg_2mi'].values
        # u,v=wind_components(aws_2minspd,aws_2mindir)

        validflag=[]
        for nn in range(len(aws_lats)):
            if aws_2minspd[nn] > 999000 or aws_2mindir[nn] > 999000 or aws_2minspd[nn] < 1:
                continue
            else:
                validflag.append(nn)
        aws_2minspd = aws_2minspd[validflag]*units('m/s')
        aws_2mindir = aws_2mindir[validflag]*units.degree
        oa_class = Object_Analyst()
        oa_class.set_reso(0.01)
        vtx_div = oa_class.calc_vor_div(aws_2minspd,aws_2mindir,aws_lons,aws_lats)

        levels = list(range(-80, 80, 2))

        cntr = self.ax1.contour(vtx_div.lon.values,vtx_div.lat.values,vtx_div.div_10m.values,
                                levels=levels,colors=self.contour_color,zorder=zorder,linewidths=self.linewidth_cntr)
        
        plt.setp(cntr.collections, path_effects=[
            patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
        clbls = self.ax1.clabel(cntr,fmt="%2.1f",use_clabeltext=True,fontsize=self.fontsize_colorbar)

        plt.setp(cntr.labelTexts, path_effects=[
            patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])

    # 画温度场客观分析等值线
    def draw_temperature(self,zorder=2):
        if self.aws_min_file_name is None:
            print('warning: aws_min_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_file_path + os.sep + self.aws_min_file_name):
            print(self.aws_min_file_path + os.sep + self.aws_min_file_name + ' does not exists!')
            return False
        awsdata = pd.read_csv(self.aws_min_file_path + os.sep + self.aws_min_file_name,encoding='GBK')

        aws_lats=awsdata['Lat'].values
        aws_lons=awsdata['Lon'].values
        aws_tem = awsdata['TEM'] # 温度
        aws_dpt = awsdata['DPT'] # 露点

        validflag=[]
        for nn in range(len(aws_lats)):
            if aws_tem[nn] > 999000 :
                continue
            else:
                validflag.append(nn)
        aws_tem = aws_tem[validflag]
        aws_lons = aws_lons[validflag]
        aws_lats = aws_lats[validflag]

        oa_class = Object_Analyst()
        oa_class.set_reso(0.01)
        params={}
        params['in_lon'] = aws_lons
        params['in_lat'] = aws_lats
        params['in_data'] = aws_tem
        params['out_varname'] = 't2m'
        params['out_long_name'] = 'surface temperature objective analyse'
        params['out_short_name'] = 'oa_t'
        params['out_units'] = 'degC'
        
        t2m = oa_class.do_oa_base(params)

        # levels = list(range(-40, 50, 0.5))
        levels = np.arange(-40,50,0.5)


        cntr = self.ax1.contour(t2m.lon.values,t2m.lat.values,t2m.t2m.values,
                                levels=levels,colors='k',zorder=zorder)
        
        plt.setp(cntr.collections, path_effects=[
            patheffects.withStroke(linewidth=2, foreground="w")])
        clbls = self.ax1.clabel(cntr,fmt="%2.1f",use_clabeltext=True,fontsize=self.fontsize_colorbar)

        plt.setp(cntr.labelTexts, path_effects=[
            patheffects.withStroke(linewidth=2, foreground="w")])


    # 画露点温度场客观分析等值线
    def draw_drewpoint(self,zorder=2):
        if self.aws_min_file_name is None:
            print('warning: aws_min_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_file_path + os.sep + self.aws_min_file_name):
            print(self.aws_min_file_path + os.sep + self.aws_min_file_name + ' does not exists!')
            return False
        awsdata = pd.read_csv(self.aws_min_file_path + os.sep + self.aws_min_file_name,encoding='GBK')

        aws_lats=awsdata['Lat'].values
        aws_lons=awsdata['Lon'].values
        aws_tem = awsdata['TEM'] # 温度
        aws_dpt = awsdata['DPT'] # 露点

        validflag=[]
        for nn in range(len(aws_lats)):
            if aws_dpt[nn] > 999000 :
                continue
            else:
                validflag.append(nn)
        aws_dpt = aws_dpt[validflag]
        aws_lons = aws_lons[validflag]
        aws_lats = aws_lats[validflag]

        oa_class = Object_Analyst()
        oa_class.set_reso(0.01)
        params={}
        params['in_lon'] = aws_lons
        params['in_lat'] = aws_lats
        params['in_data'] = aws_dpt
        params['out_varname'] = 'dpt2m'
        params['out_long_name'] = 'surface drew temperature objective analyse'
        params['out_short_name'] = 'oa_dpt'
        params['out_units'] = 'degC'
        
        dpt2m = oa_class.do_oa_base(params)

        levels = np.arange(-40,50,0.5)
       
        cntr = self.ax1.contour(dpt2m.lon.values,dpt2m.lat.values,dpt2m.dpt2m.values,
                                levels=levels,colors='k',zorder=zorder)
        
        plt.setp(cntr.collections, path_effects=[
            patheffects.withStroke(linewidth=2, foreground="w")])
        clbls = self.ax1.clabel(cntr,fmt="%2.1f",use_clabeltext=True,fontsize=self.fontsize_colorbar)

        plt.setp(cntr.labelTexts, path_effects=[
            patheffects.withStroke(linewidth=2, foreground="w")])

    def draw_vortex(self,zorder=2):
        if self.aws_min_file_name is None:
            print('warning: aws_min_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_file_path + os.sep + self.aws_min_file_name):
            print(self.aws_min_file_path + os.sep + self.aws_min_file_name + ' does not exists!')
            return False
        awsdata = pd.read_csv(self.aws_min_file_path + os.sep + self.aws_min_file_name,encoding='GBK')

        aws_lats=awsdata['Lat'].values
        aws_lons=awsdata['Lon'].values
        aws_2minspd=awsdata['WIN_S_Avg_2mi'].values
        aws_2mindir=awsdata['WIN_D_Avg_2mi'].values

        validflag=[]
        for nn in range(len(aws_lats)):
            if aws_2minspd[nn] > 999000 or aws_2mindir[nn] > 999000 or aws_2minspd[nn] < 1:
                continue
            else:
                validflag.append(nn)
        aws_2minspd = aws_2minspd[validflag]*units('m/s')
        aws_2mindir = aws_2mindir[validflag]*units.degree
        oa_class = Object_Analyst()
        oa_class.set_reso(0.01)
        vtx_div = oa_class.calc_vor_div(aws_2minspd,aws_2mindir,aws_lons,aws_lats)
        kk=0
        levels = list(range(-80, 80, 2))
      
        cntr = self.ax1.contour(vtx_div.lon.values,vtx_div.lat.values,vtx_div.vtx_10m.values,
                                levels=levels,colors='k',zorder=zorder)
        for artist in cntr.get_children():
            if isinstance(artist, matplotlib.collections.PathCollection):
                artist.set_path_effects([patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
                clbls = self.ax1.clabel(artist,fmt="%2.1f",use_clabeltext=True,fontsize=self.fontsize_colorbar)
                
        plt.setp(cntr.labelTexts, path_effects=[
            patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
        
    def draw_ppi(self,fieldname='reflectivity',tilt=0,colorfile=None,**kwargs):
        cmapdic = parse_pro(colorfile)
        if not isinstance(fieldname,str):
            print('fieldname should be string')
            return False
        if not isinstance(tilt,int):
            print('tilt should be int and >=0') 
            return False
        
        
        self.display.plot_ppi_map(fieldname, tilt,cmap=cmapdic['cmap'],gatefilter=self.gatefilter_ref,
                            min_lon=self.wlon, max_lon=self.elon, norm=cmapdic['norm'],ax=self.ax1,
                            min_lat=self.slat, max_lat=self.nlat,title_flag=False,
                            fig=self.fig, lat_0=self.g_rad_lat,lon_0=self.g_rad_lon,colorbar_flag=False,
                            **kwargs)

    def draw_wind_barb(self,zorder=1):
        # 画风场
        # filepath_minute='temp/backup_aws_minute'
        # filename_aws='surface_aws_20210720_1800.csv'
        if self.aws_min_file_name is None:
            print('warning: aws_min_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_file_path + os.sep + self.aws_min_file_name):
            print(self.aws_min_file_path + os.sep + self.aws_min_file_name + ' does not exists!')
            return False
        
        awsdata = pd.read_csv(self.aws_min_file_path + os.sep + self.aws_min_file_name,encoding='GBK')

        aws_lats=awsdata['Lat'].values
        aws_lons=awsdata['Lon'].values
        aws_2minspd=awsdata['WIN_S_Avg_2mi'].values
        aws_2mindir=awsdata['WIN_D_Avg_2mi'].values
        u,v=wind_components(aws_2minspd*units('m/s'),aws_2mindir*units.deg)
        for nn in range(len(aws_lats)):
            if aws_2minspd[nn] > 999000 or aws_2mindir[nn] > 999000 or aws_2minspd[nn] < 1:
                continue
            u,v=wind_components(float(2)*units('m/s'),int(aws_2mindir[nn])*units.deg)
            plt.barbs(aws_lons[nn],aws_lats[nn],u.magnitude*2.5,v.magnitude*2.5,length=5,barbcolor='black',linewidth=self.wind_bar_width_outter,zorder=zorder) 
            plt.barbs(aws_lons[nn],aws_lats[nn],u.magnitude*2.5,v.magnitude*2.5,length=5,barbcolor='white',linewidth=self.wind_bar_width_inner,zorder=zorder) 

    
    def draw_wind_quiver(self,zorder=1):
        
        if self.aws_min_file_name is None:
            print('warning: aws_min_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_file_path + os.sep + self.aws_min_file_name):
            print(self.aws_min_file_path + os.sep + self.aws_min_file_name + ' does not exists!')
            return False
        
        awsdata = pd.read_csv(self.aws_min_file_path + os.sep + self.aws_min_file_name,encoding='GBK')

        aws_lats=awsdata['Lat'].values
        aws_lons=awsdata['Lon'].values
        aws_2minspd=awsdata['WIN_S_Avg_2mi'].values
        aws_2mindir=awsdata['WIN_D_Avg_2mi'].values
        u,v=wind_components(aws_2minspd*units('m/s'),aws_2mindir*units.deg)
        for nn in range(len(aws_lats)):
            if aws_2minspd[nn] > 999000 or aws_2mindir[nn] > 999000 or aws_2minspd[nn] < 1:
                continue
            u,v=wind_components(float(2)*units('m/s'),int(aws_2mindir[nn])*units.deg)
           
            self.ax1.quiver(aws_lons[nn],aws_lats[nn],u.magnitude,v.magnitude,width=0.003,minshaft=1,scale=60,facecolor=[0,0,0],
                       path_effects=[path_effects.Stroke(linewidth=0.5, foreground='white'),path_effects.Normal()],label='地面2分钟平均风',
                       zorder=zorder)#


    def draw_pre_1h(self,):
        # 叠加1小时降水场
        # filepath_hour = 'temp/backup_aws_hourly'
        # filename_hour = 'surface_aws_hourly_20210720_1000.csv'

        # 方式2：从文件读取数据
        awsdata_hourly = pd.read_csv(self.aws_hour_file_path + os.sep + self.aws_hour_file_name,encoding='GBK')
        awsdata_hourly = awsdata_hourly.sort_values(by='PRE_1h')
        pre_1h = awsdata_hourly['PRE_1h'].values
        pre_lat = awsdata_hourly['Lat'].values
        pre_lon = awsdata_hourly['Lon'].values

        geodetic_transform = ccrs.Geodetic()._as_mpl_transform(self.ax1)
        text_transform = offset_copy(geodetic_transform, units='dots', x=-5)
        for nn in range(len(pre_1h)):
            if pre_1h[nn] > 1000 or pre_1h[nn] < self.thred_pre1h:
                continue
            precolor=[1,1,1]
            # print(pre_1h[nn])
            if pre_1h[nn] >= 50 and pre_1h[nn] < 80:
                precolor=[0,1,1]
            elif pre_1h[nn] >= 80 and pre_1h[nn] < 100:
                precolor=[1,1,0]
            elif pre_1h[nn] >= 100:
                precolor=[1,0,1]
            self.ax1.text(pre_lon[nn],pre_lat[nn],'%d'%(int(pre_1h[nn])), clip_on=True,
                            verticalalignment='center', horizontalalignment='right',
                            transform=text_transform, fontproperties=self.font_colorbar, color=precolor,
                            path_effects=[path_effects.Stroke(linewidth=1, foreground='black'),path_effects.Normal()])

    def draw_colorbar(self,colorfile,tickstep=5,orientation='vertical',cb_ratio=0.618):
        # 画色标
        
        # 水平色标
        if orientation == 'horizontal':
            # cbar_axes = [0.01, 0.01, 0.95, 0.04]
            cbar_axes = [self.map_panel_axes[0],self.map_panel_axes[1]+0.01+self.map_panel_axes[3],self.map_panel_axes[2]*0.618,0.03]
            
            ax2 = self.fig.add_axes(cbar_axes)
            ax2.set_axis_off()
            cmapdic = parse_pro(colorfile)
            cbar = ColorbarBase(ax2, orientation="horizontal", norm=cmapdic['norm'], cmap=cmapdic['cmap'],extend='both')
            cbar.set_ticks([])
            trans = transforms.blended_transform_factory(ax2.transData, ax2.transAxes)
            tick_poss=np.arange(cmapdic['norm'].vmin, cmapdic['norm'].vmax, tickstep)
            for tick_pos in tick_poss:
                ax2.vlines(tick_pos, 0, 0.2, transform=trans, color="black")
                ax2.vlines(tick_pos, 0.85, 1, transform=trans, color="black")

                ax2.text(tick_pos, 1.4, str(int(tick_pos)),va="center", ha="center", transform=trans, fontsize = self.fontsize_colorbar)
    
            ax2.text(1.02, 0.5, cmapdic['units'], transform=ax2.transAxes,va="center", ha="center",fontsize = self.fontsize_colorbar)
            

        # 垂直色标
        if orientation == 'vertical':
            ax_pos = self.ax1.get_position().bounds
            cbar_axes = [ax_pos[0]+ ax_pos[2]+0.01,(ax_pos[1]+ax_pos[1]+ax_pos[3])/2-ax_pos[3]*cb_ratio/2,0.02,ax_pos[3]*cb_ratio]
            cb_ax = self.fig.add_axes(cbar_axes)
            colordata = parse_pro(colorfile)
            
            cbar = ColorbarBase(cb_ax, orientation=orientation, norm=colordata['norm'], cmap=colordata['cmap'],extend='both')
            cbar.set_ticks([])
            trans = transforms.blended_transform_factory(cb_ax.transData, cb_ax.transAxes)
            tick_poss=np.arange(colordata['norm'].vmin, colordata['norm'].vmax, tickstep)
            xrange = colordata['norm'].vmax - colordata['norm'].vmin

            for tick_pos in tick_poss:
                cb_ax.hlines(tick_pos, colordata['norm'].vmin, colordata['norm'].vmin + 0.3*xrange, color="black",linewidth=0.5)
                cb_ax.hlines(tick_pos, colordata['norm'].vmin + 0.7*xrange, colordata['norm'].vmax, color="black",linewidth=0.5)
    

                cb_ax.text(1.04,tick_pos, str(int(tick_pos)),va="center", ha="left",fontsize=5) # font=self.font_colorbar

            cb_ax.text(1.85, 1.035, colordata['units'], transform=cb_ax.transAxes,va="center", ha="center", fontsize=5 ) # ,font=self.font_colorbar
                       
        
    
    def draw_gisinfo(self,):
        geodetic_transform = ccrs.Geodetic()._as_mpl_transform(self.ax1)
        text_transform = offset_copy(geodetic_transform, units='dots', x=-5)
        aa = (np.array(self.gis_lats)<self.nlat) & (np.array(self.gis_lats) > self.slat)
        bb = (np.array(self.gis_lons)<self.elon) & (np.array(self.gis_lons) > self.wlon)
        cc = aa & bb

        alltxt=[]
        for nn in range(len(self.gis_name)):
            if not cc[nn]:
                continue
            curlat = self.gis_lats[nn]
            curlon = self.gis_lons[nn]
            self.ax1.plot(curlon, curlat, marker='o', color=[0.2,0.2,0.2], linestyle='None',
                markersize=1, alpha=0.8, transform=ccrs.PlateCarree())
            txt_lon = curlon
            txt_lat = curlat
            texts = self.ax1.text(txt_lon, txt_lat, self.gis_name[nn], clip_on=True,
                            verticalalignment='center', horizontalalignment='right',
                            transform=text_transform, fontproperties=self.font_gis, color='white',
                            path_effects=[path_effects.Stroke(linewidth=1, foreground='black'),path_effects.Normal()])


    def add_china_map_2cartopy(self, ax, name='province',edgecolor='k', lw=0.8,facecolor='none',transform=None, **kwargs):
        """
        Draw china boundary on cartopy map.

        :param ax: matplotlib axes instance.
        :param name: map name.
        :param facecolor: fill color, default is none.
        :param edgecolor: edge color.
        :param lw: line width.
        :return: None
        """

        # map name
        names = {'nation': "bou1_4p", 'province': "bou2_4p",
                'county': "BOUNT_poly", 'river': "hyd1_4l",
                'river_high': "hyd2_4l"}

        # get shape filename
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"

        # add map
        ax.add_geometries(
            Reader(shpfile).geometries(), ccrs.PlateCarree(),
            path_effects=[path_effects.Stroke(linewidth=lw+0.3, foreground='white'),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw, **kwargs)

    def draw_basemap(self,):
        # Setting projection and ploting the second tilt
        projection = ccrs.PlateCarree()
        
        self.fig = plt.figure(figsize=(self.paintsize_x,self.paintsize_y))

        self.map_panel_axes = [0.12, 0.1, .76, .8]
        self.ax1 = self.fig.add_axes(self.map_panel_axes, projection=projection)
        pass

    def add_chinamap(self,):
        self.add_china_map_2cartopy(self.ax1, name='county',edgecolor=self.mapcolor, lw=self.linewidth_map)


    def draw_title(self,titlestr='XX雷达0.5度仰角径向速度'):
        if not self.bdraw_title_ppi:
            return None
        
        tstr=''
        if self.voltime is None:
            tstr = self.timestr
        else:
            tstr = self.voltime

        plt.title('%s(%s BJT)'%(titlestr,tstr),fontproperties=self.font_title,loc='right',verticalalignment='top')

    def crs_finish_save(self,bshow=False,subdir='',add_str='',):

        
        try:
            if not os.path.exists(self.picpath + os.sep + subdir): 
                os.makedirs(self.picpath + os.sep + subdir)
        except:
            pass
        if self.pic_format=='ps':
            plt.savefig(self.picpath + os.sep + subdir + os.sep + self.timestr + '.' + add_str + '.' + self.pic_format,dpi=self.dpi)
        else:
            plt.savefig(self.picpath + os.sep + subdir + os.sep + self.timestr + '.' + add_str + '.' + self.pic_format,dpi=self.dpi,bbox_inches='tight')

        print(self.picpath + os.sep + subdir + os.sep + self.timestr + '.' + add_str + '.' + self.pic_format)
        if bshow:
            plt.show()

        plt.close('all')

    def finish_save(self,bshow=False,subdir='',):
        self.ax1.axis('tight')
        self.ax1.set_xlim([self.wlon,self.elon])
        self.ax1.set_ylim([self.slat,self.nlat])

        if self.elon - self.wlon > 5:
            lon_step = 1
        elif self.elon - self.wlon > 1:
            lon_step = 0.5
        else:
            lon_step = 0.2

        if self.nlat - self.slat > 5:
            lat_step = 1
        elif self.nlat - self.slat > 1:
            lat_step = 0.5
        else:
            lat_step = 0.2
        self.ax1.xaxis.set_major_formatter(major_formatter_lon)
        self.ax1.yaxis.set_major_formatter(major_formatter_lat)
        self.ax1.set_xticks(np.arange(self.wlon,self.elon,lon_step))
        self.ax1.set_yticks(np.arange(self.slat,self.nlat,lat_step))

        self.ax1.minorticks_on()

        # self.ax1.legend(loc='upper right')
        if not os.path.exists(self.picpath + os.sep + subdir): os.makedirs(self.picpath + os.sep + subdir)
        if self.pic_format=='ps':
            plt.savefig(self.picpath + os.sep + subdir + os.sep + self.timestr + '.' + self.pic_format,dpi=self.dpi)
        else:
            plt.savefig(self.picpath + os.sep + subdir + os.sep + self.timestr + '.' + self.pic_format,dpi=self.dpi,bbox_inches='tight')
        print(self.picpath + os.sep + subdir + os.sep + self.timestr + '.' + self.pic_format)
        if bshow:
            plt.show()

        plt.close('all')

    def draw_vel_pre(self,subdir='径向速度+当前小时降水',tilt=1):
        # check 
        if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='velocity',tilt=tilt,colorfile=self.vel_colorfile)
        self.draw_pre_1h()
        self.add_chinamap()
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.vel_colorfile)
        self.finish_save(subdir=subdir)

    def draw_ref_alone(self,subdir='基本反射率',tilt=0,thred=-5,):
        # check 
        if self.radar is None:
            return False
        self.gatefilter_ref = pyart.correct.GateFilter(self.radar)
        self.gatefilter_ref.exclude_below('reflectivity', thred, exclude_masked=True, op='or', inclusive=False)

        ele=np.mean(self.radar.get_elevation(tilt))
        subdir='%.1f_%s'%(ele,subdir)
        if not self.breplace and os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
            print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.add_chinamap()
        self.draw_title(subdir)
        self.draw_colorbar(colorfile=self.ref_colorfile,orientation='vertical')
        self.add_crs_line(ax=self.ax1)
        
        self.finish_save(subdir=subdir,)

    # 画相关系数，偏振量
    def draw_cc_alone(self,subdir='相关系数',tilt=0,thred=-5,):
        # check 
        if self.radar is None:
            return False
        self.gatefilter_ref = pyart.correct.GateFilter(self.radar)
        self.gatefilter_ref.exclude_below('cross_correlation_ratio', thred, exclude_masked=True, op='or', inclusive=False)

        ele=np.mean(self.radar.get_elevation(tilt))
        subdir='%.1f_%s'%(ele,subdir)
        if not self.breplace and os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
            print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='cross_correlation_ratio',tilt=tilt,colorfile=self.ref_colorfile)
        self.add_chinamap()
        self.draw_title(subdir)
        self.draw_colorbar(colorfile=self.cc_colorfile,orientation='vertical')
        self.add_crs_line(ax=self.ax1)
        
        self.finish_save(subdir=subdir,)


    # 画差分反射率，偏振量
    def draw_zdr_alone(self,subdir='差分反射率',tilt=0,thred=-5,):
        # check 
        if self.radar is None:
            return False
        self.gatefilter_ref = pyart.correct.GateFilter(self.radar)
        self.gatefilter_ref.exclude_below('cross_correlation_ratio', thred, exclude_masked=True, op='or', inclusive=False)

        ele=np.mean(self.radar.get_elevation(tilt))
        subdir='%.1f_%s'%(ele,subdir)
        if not self.breplace and os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
            print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='cross_correlation_ratio',tilt=tilt,colorfile=self.ref_colorfile)
        self.add_chinamap()
        self.draw_title(subdir)
        self.draw_colorbar(colorfile=self.zdr_colorfile,orientation='vertical')
        self.add_crs_line(ax=self.ax1)
        
        self.finish_save(subdir=subdir,)

    def draw_vel_alone(self,subdir='径向速度',tilt=0,format='png'):
        # check 
        if self.radar is None:
            return False
        ele=np.mean(self.radar.get_elevation(tilt))
        subdir='%.1f_%s'%(ele,subdir)
        
        if not self.breplace and os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.' + format):
            print(self.picpath + os.sep + subdir + self.timestr + '.' + format + ' already exists!')
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='velocity',tilt=tilt,colorfile=self.vel_colorfile)
        self.add_chinamap()
        self.draw_title(subdir)
        self.add_crs_line(ax=self.ax1)
        self.draw_colorbar(colorfile=self.vel_colorfile)
        self.finish_save(subdir=subdir,)

    def draw_vel_wind_barb(self,subdir='径向速度+地面自动站风场',tilt=1):
        # check 
        if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
            print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='velocity',tilt=tilt,colorfile=self.vel_colorfile)
        self.add_chinamap()
        self.draw_wind_barb()
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.vel_colorfile)
        self.finish_save(subdir=subdir)

    def draw_vel_wind_quiver(self,subdir='径向速度+地面流场',tilt=1):
        # check 
        if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
            print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
            return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='velocity',tilt=tilt,colorfile=self.vel_colorfile)
        self.add_chinamap()
        self.draw_wind_quiver()
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.vel_colorfile)
        self.finish_save(subdir=subdir)

    def draw_ref_pre(self,subdir='基本反射率+当前小时降水',tilt=0):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.draw_pre_1h()
        self.add_chinamap()
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)

    def draw_cref_pre(self,subdir='组合反射率+当前小时降水'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_cref()
        self.draw_pre_1h()
        self.add_chinamap()
        self.draw_title('组合反射率因子+当前小时降水量')
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)

    def draw_cref_wind_barb(self,subdir='组合反射率+地面自动站风场'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_cref()
        self.add_chinamap()
        self.draw_wind_barb()
        self.draw_title(subdir)
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)

    def draw_cref_wind_quiver(self,subdir='组合反射率+地面流场'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_cref()
        self.add_chinamap()
        self.draw_wind_quiver()
        self.draw_title(subdir)
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)

    def draw_cref_pre_wind_barb(self,subdir='组合反射率+地面自动站风场+当前小时降水'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False

        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_cref()
        self.draw_wind_barb()
        self.draw_pre_1h()
        self.add_chinamap()
        self.draw_title(subdir)
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)


    def draw_ref_wind_barb_vortex(self,subdir='基本反射率+地面自动站风场+涡度场(等值线)',tilt=0):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.draw_vortex(zorder=2)
        self.add_chinamap()
        self.draw_wind_barb(zorder=10)
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)
    
    def draw_ref_wind_divergence(self,subdir='基本反射率+地面自动站风场+散度(等值线)',tilt=0,type='quiver'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.draw_divergence(zorder=2)
        self.add_chinamap()
        if type == 'quiver':
            self.draw_wind_quiver(zorder=10)
        elif type == 'barb':
            self.draw_wind_barb(zorder=10)
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)

    def draw_ref_wind_barb_temperature(self,subdir='基本反射率+地面自动站风场+温度场(等值线)',tilt=0):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.draw_temperature(zorder=2)
        self.add_chinamap()
        self.draw_wind_barb(zorder=10)
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f°%s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)
    
    def draw_ref_wind_barb_dewpoint(self,subdir='基本反射率+地面自动站风场+露点场(等值线)',tilt=0):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.draw_drewpoint(zorder=2)
        self.add_chinamap()
        self.draw_wind_barb(zorder=10)
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)


    def draw_ref_pre_wind(self,subdir='基本反射率+地面自动站风场+当前小时降水',type='barb',tilt=0):
        # type == barb or quiver
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        if type == 'barb':
            self.draw_wind_barb()
        elif type == 'quiver':
            self.draw_wind_quiver()
        self.draw_pre_1h()
        self.add_chinamap()
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)

    def draw_ref_wind(self,subdir='基本反射率+地面自动站风场',type='barb',tilt=0):
        # type == barb or quiver
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.png'):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.png' + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        if type == 'barb':
            self.draw_wind_barb()
        elif type == 'quiver':
            self.draw_wind_quiver()
        self.add_chinamap()
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)


    # 叠加1小时变温
    def draw_ref_wind_delta_t(self,subdir='基本反射率+地面自动站风场+1小时变温(等值线)',tilt=0,type='quiver'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + '.' + self.pic_format):
        #     print(self.picpath + os.sep + subdir + self.timestr + '.' + self.pic_format + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        # self.draw_divergence(zorder=2)
        self.draw_delta_t(zorder=2)
        self.add_chinamap()
        if type == 'quiver':
            self.draw_wind_quiver(zorder=10)
        elif type == 'barb':
            self.draw_wind_barb(zorder=10)
        ele=np.mean(self.radar.get_elevation(tilt))
        self.draw_title('%s %.1f° %s'%(self.radarname,ele, subdir))
        self.draw_colorbar(colorfile=self.ref_colorfile)
        self.finish_save(subdir=subdir)
    
    # 叠加1小时变压
    def draw_ref_wind_delta_p(self,subdir='基本反射率+地面自动站风场+1小时变压(等值线)',tilt=0,type='quiver'):
        # check 
        # if os.path.exists(self.picpath + os.sep + subdir + os.sep +self.timestr + + '.' + self.pic_format):
        #     print(self.picpath + os.sep + subdir + self.timestr + + '.' + self.pic_format + ' already exists!')
        #     return False
        self.draw_basemap()
        self.draw_gisinfo()
        self.draw_ppi(fieldname='reflectivity',tilt=tilt,colorfile=self.ref_colorfile)
        self.draw_delta_p(zorder=2)
        
        
    def draw_delta_t(self,zorder=2):
        
        # 叠加1小时变温
        # filepath_minute='temp/backup_aws_minute'
        # filename_aws='surface_aws_20210720_1800.csv'
        if self.aws_min_delta_t_file_name is None:
            print('warning: aws_min_delta_t_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_delta_t_file_path + os.sep + self.aws_min_delta_t_file_name):
            print(self.aws_min_delta_t_file_path + os.sep + self.aws_min_delta_t_file_name + ' does not exists!')
            return False
        
        delta_t_data = xr.open_dataset(self.aws_min_delta_t_file_path + os.sep + self.aws_min_delta_t_file_name)
        

        levels = list(np.arange(delta_t_data.attrs['sta_minvalue'], delta_t_data.attrs['sta_maxvalue'], 0.2))

        cntr = self.ax1.contour(delta_t_data.lon.values,delta_t_data.lat.values,delta_t_data.t2m_delta_1hr.values.T,
                                levels=levels,colors=self.contour_color,zorder=zorder,linewidths=self.linewidth_cntr)
        
        for artist in cntr.get_children():
            if isinstance(artist, matplotlib.collections.PathCollection):
                artist.set_path_effects([patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
                clbls = self.ax1.clabel(artist,fmt="%2.1f",use_clabeltext=True,fontsize=self.fontsize_colorbar)

        plt.setp(cntr.labelTexts, path_effects=[
            patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
    
    def draw_delta_p(self,zorder=2):
        
        # 叠加1小时变压
        # filepath_minute='temp/backup_aws_minute'
        # filename_aws='surface_aws_20210720_1800.csv'
        if self.aws_min_delta_p_file_name is None:
            print('warning: aws_min_delta_p_file_name is None!')
            return False
        if not os.path.exists(self.aws_min_delta_p_file_path + os.sep + self.aws_min_delta_p_file_name):
            print(self.aws_min_delta_p_file_path + os.sep + self.aws_min_delta_p_file_name + ' does not exists!')
            return False
        
        delta_p_data = xr.open_dataset(self.aws_min_delta_p_file_path + os.sep + self.aws_min_delta_p_file_name)
        

        levels = list(np.arange(delta_p_data.attrs['sta_minvalue'], delta_p_data.attrs['sta_maxvalue'], 0.1))

        cntr = self.ax1.contour(delta_p_data.lon.values,delta_p_data.lat.values,delta_p_data.sprs2m_delta_1hr.values.T,
                                levels=levels,colors=self.contour_color,zorder=zorder,linewidths=self.linewidth_cntr,label='1小时变压')
        
        for artist in cntr.get_children():
            if isinstance(artist, matplotlib.collections.PathCollection):
                artist.set_path_effects([patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
                clbls = self.ax1.clabel(artist,fmt="%2.1f",use_clabeltext=True,fontsize=self.fontsize_colorbar,inline=True)

        plt.setp(cntr.labelTexts, path_effects=[
            patheffects.withStroke(linewidth=self.linewidth_cntr, foreground="w")])
        return cntr
    

    
        
        

if __name__ == "__main__":
    
    import pandas as pd
    import numpy as np  
    # 准备GIS文件
    gisfile = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/resources/stations/cma_city_station_info.dat'

    cities = pd.read_csv(gisfile,  delimiter=r"\s+") # cma_city_station_info

    cityname = np.unique(cities['city_name'])
    stanames=[]
    lat=[]
    lon=[]

    curdata = cities
    for name in cityname:
        flag2 =  curdata['city_name'] == name
        tmp_lat = np.mean(curdata[flag2]['lat'])
        tmp_lon = np.mean(curdata[flag2]['lon'])

        stanames.append(name)
        lon.append(tmp_lon)
        lat.append(tmp_lat)
        
        
    # 初始化绘图参数
    params=ini_params()

    # 修改补充完善参数
    params['radarfile_path'] = '/mnt/e/metradar_test/radar_aws/radar_fmt/2021/07/20/Z9371'
    params['radarfile_name'] = 'Z_RADR_I_Z9371_20210720083000_O_DOR_SAD_CAP_FMT.bin.bz2'
    params['rose_cr_path'] = '/mnt/e/metradar_test/radar_aws/pup/Z9371/CR/37'
    params['rose_cr_name'] = 'Z9371_20210720083000Z_CR_00_37'
    params['aws_hour_file_path'] = '/mnt/e/metradar_test/radar_aws/awsdata/aws_hourly_obs'
    params['aws_hour_file_name'] = 'surface_aws_hourly_20210720_0900.csv'
    params['aws_min_file_path'] = '/mnt/e/metradar_test/radar_aws/awsdata/aws_minutes_obs'
    params['aws_min_file_name'] = 'surface_aws_minute_20210720_0824_0830.csv'
    
    params['pic_path'] = '/mnt/e/metradar_test/radar_aws/pic'
    params['slat'] = 34
    params['nlat'] = 35
    params['wlon'] = 113
    params['elon'] = 114
    params['timestr'] = params['radarfile_name'].split('_')[4]
    params['fontfile'] = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/resources/fonts/YaHeiConsolasHybrid_1.12.ttf'
    params['gis_lats'] = lat
    params['gis_lons'] = lon
    params['gis_name'] = stanames
    params['breplace'] = True
    params['bdraw_crs'] = False
    params['ref_colorfile'] = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/resources/gr2_colors/default_BR_PUP2.pal'
    params['vel_colorfile'] = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/resources/gr2_colors/default_BV_PUP2.pal'
    params['figsize_width'] = 4
    params['fontsize_gis'] = 5
    params['fontsize_colorbar'] = 5
    params['fontsize_title'] = 6
    params['mapcolor'] = [0/255,0/255,0/255]
    params['dpi'] = 300
    params['pic_format'] = 'png'
    params['bdraw_title_ppi'] = False


    # 创建绘图函数
    _draw_radar_other = DRAW_RADAR_OTHER(params)

    # _draw_radar_other.read_vol_data()

    # _draw_radar_other.draw_ref_alone(subdir='ref',tilt=0,thred=-5)
    
    # _draw_radar_other.draw_vel_alone(subdir='vel',tilt=1)
    
    # _draw_radar_other.draw_ref_pre(subdir='ref_pre',)
    
    # _draw_radar_other.draw_ref_wind_barb_vortex(subdir='ref_pre_wind_vortex',)
    
    # _draw_radar_other.get_cref_from_radar(xlimits=[-80,50],ylimits=[-80,50])
    # _draw_radar_other.draw_cref_wind_barb(subdir='cref_wind_barb',)
    # _draw_radar_other.draw_cref_pre(subdir='cref_pre',)
    
    _draw_radar_other.get_cref_from_rose()
    # _draw_radar_other.draw_cref_wind_barb(subdir='cref_wind_barb',)
    _draw_radar_other.draw_cref_pre(subdir='cref_pre',)



# _*_ coding: utf-8 _*_


'''
将雨强文件转换为定量降水产品

'''

import os
from datetime import datetime,timedelta
from typing import Dict
import numpy as np
import pyart 
import configparser
import warnings
import scipy.interpolate
warnings.filterwarnings('ignore')
import pickle
from netCDF4 import Dataset
from threading import Thread
from multiprocessing import cpu_count
#添加时间控制器
import time
import sched
import sys
schedule = sched.scheduler(time.time, time.sleep)

from multiprocessing import Pool,cpu_count,freeze_support

BDEBUG = False
# sub function for reading config file
def ConfigFetchError(BaseException):
    pass

def _get_config_from_rcfile(rcfile):
    """
    Get configure information from config_dk_met_io.ini file.
    """

    print(rcfile)
    try:
        config = configparser.ConfigParser()
        config.read(rcfile,encoding='UTF-8')
    except IOError as e:
        raise ConfigFetchError(str(e))
    except Exception as e:
        raise ConfigFetchError(str(e))

    return config

def get_datetime_from_filename1(filename):
    timstr = filename[15:29]
    fyear = int(timstr[0:4])
    fmonth = int(timstr[4:6])
    fday = int(timstr[6:8])
    fhour = int(timstr[8:10])
    fmin = int(timstr[10:12])
    fsec = int(timstr[12:14])
    ft = datetime(fyear,fmonth,fday,fhour,fmin,fsec)
    return ft

class MyThread(Thread):
 
  def __init__(self, fullfilepath):
    Thread.__init__(self)
    self.filepath = fullfilepath
 
  def run(self):
    self.result = pyart.io.read_grid(self.filepath)
 
  def get_result(self):
    return self.result

def do_single(params:Dict):
    # 历史模式下，需要将所有文件都处理
    curt = params['curt']
    rd = params['rd']
    acc_hours = params['acc_hours']
    allfiles = params['allfiles']
    curoutpath = params['curoutpath']
    GRID_SHAPE = params['GRID_SHAPE']
    GRID_LIMITS = params['GRID_LIMITS']
    mosaic_lons = params['mosaic_lons']
    mosaic_lats = params['mosaic_lats']
    glons = params['glons']
    glats = params['glats']
    print(rd + ' 站当前最新的数据时间为: ' + curt.strftime('%Y-%m-%d %H:%M:%S'))
    td = timedelta(hours=acc_hours)
    pret = curt - td
    files=[]
    for ff in allfiles:
        ft = get_datetime_from_filename1(ff)
        if ft < pret or ft > curt:
            continue
        else:
            files.append(ff)
            # print(ff)

    outrain = np.zeros([len(files),GRID_XNUM,GRID_YNUM],dtype=float)

    # 定义时间戳列表
    tlist = []
    valid_filenum = 0
    rad_lon = 0
    rad_lat = 0
    rad_alti = 0
    rad_name = ''
    
    for filename in files:
        # print(filename)
        try:
            grid = pyart.io.read_grid(PATH_RR + os.sep + rd + os.sep + filename)
            pass
        except:
            print(PATH_RR + os.sep + rd + os.sep + filename + '读取失败，暂不处理！')
            try:
                os.remove(PATH_RR + os.sep + rd + os.sep + filename)
                print(PATH_RR + os.sep + rd + os.sep + filename + ' removed!')
            except:
                print('delete file failed! ' + filename)
                pass
            files.remove(filename)
            continue
        outrain[valid_filenum,:,:] = grid.fields['radar_estimated_rain_rate']['data'][0,:,:]
        dateTime_p = datetime.strptime(grid.time['units'][14::],'%Y-%m-%dT%H:%M:%SZ')
        tlist.append(dateTime_p)
        rad_lon = grid.radar_longitude['data']
        rad_lat = grid.radar_latitude['data']
        rad_alti = grid.radar_altitude['data']
        # rad_name = grid.metadata['instrument_name']
        valid_filenum +=1
        
    # 将最后一个文件的文件名中的时间作为当前站点qpe的时间时，不可以执行下一行    
    # tlist.append(curt) # 最后一个文件的权重由该文件的时间到当前规定的累积截止时间之间的差值
    # print('%s 站，有效文件数为: %d ,本站处理结束！'%(rd,valid_filenum))   
    # print('%s 站共找到 %d 个有效文件'%(rd,valid_filenum))
    outfilename = rd + '_' + curt.strftime('%Y%m%d_%H%M%S_') + '%02dhr.nc'%acc_hours

    curn = rd + '_%02dhr'%acc_hours + curt.strftime('_%Y%m%d_%H%M%S')
    bneed_process = True
    if os.path.exists(curoutpath + os.sep + outfilename):
        #如果目标文件已经存在，那么check它的源数据是否有更新，如果有更新，那么重新累加，如果没有，那么不累加，直接跳过

        if os.path.exists(PATH_TMP + os.sep + curn ):
            if os.path.getsize(PATH_TMP + os.sep + curn ) == 0:
                raise ValueError(f"文件 {PATH_TMP + os.sep + curn } 为空")
            try:
                f = open(PATH_TMP + os.sep + curn,'rb')
                dic_info = pickle.load(f)
                f.close()
                print()
            except ValueError as e:
                print(f"读取文件 {PATH_TMP + os.sep + curn } 时出错: {e}")
                dic_info = None
                
        if isinstance(dic_info,str):
            if set(tlist) == set(dic_info):
                print(curn + ' 源数据无变化，不重复处理！')
                bneed_process = False
            else:
                pass
        elif isinstance(dic_info,list):
           
            if set(tlist) == set(dic_info):
                print(curn + ' 源数据无变化，不重复处理！')
                bneed_process = False
            else:
                pass
        
    else:
        pass
    # 如果无需重复处理，就直接continue，进入下一站
    if not bneed_process:
        return True
    f = open(PATH_TMP + os.sep + curn,'wb')
    pickle.dump(tlist,f)
    f.close()
    #计算时间权重
    dt = list(np.diff(tlist))
    
    # 插入第一个文件的时间权重，限定一个范围，不要超过6分钟
    
    dt.insert(0,min(timedelta(seconds = acc_hours * 3600 - round(np.sum([gk.seconds for gk in dt]))),timedelta(seconds=360)))
    
    # print(dt)

    acs = 0
    wgts = []
    for tt in dt:
        acs += tt.seconds
        wgts.append(tt.seconds / (1* 3600)) # 由于计算的是小时雨强，因此，这里算权重的时候只能按1小时来算，即3600秒
    if acs > (acc_hours * 3600):
        # print('累积时段和设置的不一样，设置的时段是 %d 小时，当前计算的是 %.1f 小时'%(acc_hours,acs/3600))
        print('累积时段超过了设置的规定时段，设置的时段是 %d 小时，当前累积时长是 %.2f 小时'%(acc_hours,acs/3600))
    # print(wgts)


    # 累加降水
    acc_rain = np.zeros([GRID_XNUM,GRID_YNUM],dtype=float)

    for fn in np.arange(0,valid_filenum):
        acc_rain += np.reshape(outrain[fn],[GRID_XNUM,GRID_YNUM])*wgts[fn]

    

    # 将rain插值到大的网格上，方便下一步直接调用
    nz, ny, nx = GRID_SHAPE
    (z0, z1), (y0, y1), (x0, x1) = GRID_LIMITS
    g_lons,g_lats = pyart.core.transforms.cartesian_to_geographic_aeqd(
                                                            np.linspace(x0, x1, nx),
                                                            np.linspace(y0, y1, ny),
                                                            rad_lon,
                                                            rad_lat)
    mesh_lons,mesh_lats = np.meshgrid(g_lons,g_lats)
    mosaic_grid  = scipy.interpolate.griddata((mesh_lons.flatten(),mesh_lats.flatten()),acc_rain.flatten(),(glons,glats))

    # %% 输出到nc格式
    try:
        ncfile = Dataset(curoutpath + os.sep + outfilename + '.lock', 'w', format='NETCDF4')
    except:
        if os.path.exists(curoutpath + os.sep + outfilename+ '.lock'): 
            try:
                os.remove(curoutpath + os.sep + outfilename+ '.lock')
                ncfile = Dataset(curoutpath + os.sep + outfilename+ '.lock', 'w', format='NETCDF4')
            except:
                print(curoutpath + os.sep + outfilename +  '.lock' + ' 文件创建失败，删除成功，但无法再新建，跳过！')
                return False
        else:
            print(curoutpath + os.sep + outfilename + '.lock' + ' 文件创建失败，程序跳过！')
            return False
    
    used_hours = (curt.timestamp() - get_datetime_from_filename1(files[0]).timestamp())/3600
    used_hours = float('%.1f'%used_hours)

    hreso = MOSAIC_RESO
            
    # global
    ncfile.description = 'SINGLE RADAR QPE MOSAIC'
    ncfile.author = 'Zhu WenJian, kevin2075@163.com'
    ncfile.reso = hreso
    ncfile.sitenum = rd
    ncfile.radlon = rad_lon
    ncfile.radlat = rad_lat
    ncfile.start_lat = START_LAT
    ncfile.end_lat = END_LAT
    ncfile.start_lon = START_LON
    ncfile.end_lon = END_LON
    ncfile.product_time = curt.strftime('%Y%m%d%H%M') 
    ncfile.last_update = datetime.utcnow().strftime('%Y%m%d%H%M') 
    ncfile.basefile_info = files
    ncfile.used_hours = used_hours
    ncfile.acc_hours = acc_hours

    # dimensions
    nlons = mosaic_lons
    nlats = mosaic_lats

    ncfile.createDimension('longitude', len(nlons))
    ncfile.createDimension('latitude', len(nlats))

    # variables
    longitude = ncfile.createVariable('longitude', np.dtype('float32').char, ('longitude',))
    longitude.units = 'degrees_east'
    longitude.long_name = 'Longitude'
    longitude.CoordinateAxisType = 'Lon'

    latitude = ncfile.createVariable('latitude', np.dtype('float32').char, ('latitude',))
    latitude.units = 'degrees_north'
    latitude.long_name = 'Latitude'
    latitude.CoordinateAxisType = 'Lat'

    # H	unsigned short
    qpe_mosaic = ncfile.createVariable('qpe_mosaic', 'H', ('latitude', 'longitude'), zlib=True, least_significant_digit=1)
    qpe_mosaic.unit_scale = 0.1
    qpe_mosaic.long_name = "QPE MOSAIC SINGLE RADAR"
    qpe_mosaic.units='mm'
    # data
    qpe_mosaic[:,:] = mosaic_grid * 10
    longitude[:] = nlons
    latitude[:] = nlats

    ncfile.close()
    os.rename(curoutpath + os.sep + outfilename + '.lock',curoutpath + os.sep + outfilename)
    print(curoutpath + os.sep + outfilename + ' done!')
    return 0


def sub_func(params:tuple):
    acc_hours,rd = params
    curfilepath = PATH_RR + os.sep + rd
    curoutpath = PATH_QPE + os.sep + rd + os.sep

    GRID_SHAPE = (1,GRID_XNUM, GRID_YNUM)
    GRID_LIMITS = ((GRID_RESO, GRID_RESO), (-1000*(GRID_XNUM-1)/2, 1000*(GRID_XNUM-1)/2), (-1000*(GRID_XNUM-1)/2, 1000*(GRID_XNUM-1)/2))


    mosaic_lats = np.arange(START_LAT,END_LAT+MOSAIC_RESO/2,MOSAIC_RESO)
    mosaic_lons = np.arange(START_LON,END_LON+MOSAIC_RESO/2,MOSAIC_RESO)
    glons,glats = np.meshgrid(mosaic_lons,mosaic_lats)

    if not os.path.exists(curoutpath):

        os.makedirs(curoutpath,exist_ok=True)

    allfiles = os.listdir(curfilepath)
    allfiles = sorted(allfiles)
    if len(allfiles) == 0:
        return False
    for ff in allfiles:
        if ff.find('.nc') < 0 or ff.find('.lock') > 0:
            allfiles.remove(ff)
      
    # print('%s 站共找到 %d 个文件.'%(rd,len(allfiles)))

    # 开始累积降水
    # 将最后一个文件的文件名中的时间作为当前站点qpe的时间
    #设置当前截止时间
    
    #实时模式下，用最新的即可
    if RUNMODE ==0:
        curttime = get_datetime_from_filename1(allfiles[-1])
        params={}
        params['curt'] = curttime
        params['rd'] = rd
        params['acc_hours'] = acc_hours
        params['allfiles'] = allfiles
        params['curoutpath'] = curoutpath
        params['GRID_SHAPE'] = GRID_SHAPE
        params['GRID_LIMITS'] = GRID_LIMITS
        params['mosaic_lons'] = mosaic_lons
        params['mosaic_lats'] = mosaic_lats
        params['glons'] = glons
        params['glats'] = glats
        
        do_single(params)
    #历史模式下，需要将所有文件都处理
    else:
        allpms=[]
        freeze_support()
        pools = Pool(CPU_MAX)
        for f in allfiles:
            curt = get_datetime_from_filename1(f)
            params={}

            params['curt'] = curt
            params['rd'] = rd
            params['acc_hours'] = acc_hours
            params['allfiles'] = allfiles
            params['curoutpath'] = curoutpath
            params['GRID_SHAPE'] = GRID_SHAPE
            params['GRID_LIMITS'] = GRID_LIMITS
            params['mosaic_lons'] = mosaic_lons
            params['mosaic_lats'] = mosaic_lats
            params['glons'] = glons
            params['glats'] = glats
            # pools.apply_async(do_single,(params,))
            allpms.append(params)
        pools.map(do_single,allpms)
        pools.close()
        pools.join()

def get_qpe_from_rr(tstep=10):
    #设置累积时段
    # acc_hours = 3
    # RADARS = ['Z9734','Z9731','Z9739','Z9746']
    # PATH_RR = 'rain_rate'
    # PATH_QPE = 'out_qpe'
    #降水产品格点设置
    # GRID_XNUM = 701 #X方向格点数
    # GRID_YNUM = 701 #Y方向格点数
    # GRID_RESO = 1000 #网格分辨率，米

    # 用一个状态变量记录当前处理的数据相关信息，便于对比，减少重复计算量
    
  
    if len(ACC_HOURS)==0:
        print('累计时段为0小时，程序退出！')
        return False
    
    # 如果是实时模式
    if RUNMODE == 0:
        params=[]
        for acc_hours in ACC_HOURS: 
            for rd in RADARS:
                params.append((acc_hours,rd))

        freeze_support()
        print("CPUS = %d"%CPU_MAX)
        print('正在启用%d个CPU进行并行计算，等待保存结果......'%CPU_MAX)
        pool = Pool(CPU_MAX)
        pool.map(sub_func,params)
    else:
        for acc_hours in ACC_HOURS: 
            for rd in RADARS:
                sub_func((acc_hours,rd))
            

    if not BDEBUG:
        print('Waiting for new data......')
        schedule.enter(tstep, 0, get_qpe_from_rr, (tstep,))  
    else:
        pass

def _delete_old_data(tstep):
    
    print('delete expired data ...')
    curtime = datetime.now()
    pretime = curtime + timedelta(hours=-1 * int(data_save_hours))
    for rds in RADARS:
        #查询过期数据
        curpath = PATH_QPE + os.sep + rds + os.sep
        if not os.path.exists(curpath):
            continue
        ff = os.listdir(curpath)
        for f in ff:
            t = os.path.getctime(curpath + os.sep + f)
           
            if t < pretime.timestamp():
                os.remove(curpath + os.sep + f)
                print(['Delete file:' + curpath + os.sep + f])
            
       
    schedule.enter(tstep, 0, _delete_old_data, (tstep,))

if __name__ == '__main__':

    print('Usage: python s3_trans_rainrate_to_qpe.py inifile')
    # 所有参数（包含脚本名）
    all_args = sys.argv  
    # 实际参数（排除脚本名）
    inifile = sys.argv[1:]
    

    #如果未指定配置文件，则选用默认的配置
    if len(inifile) ==0:
        inifile = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/metradar/project/qpe/main_qpe_cfg.ini'
    else:
        inifile = inifile[0]
    
    config = _get_config_from_rcfile(inifile)
    RUNMODE = int(config['COMMON']['RUN_MODE'])#X方向格点数
    if RUNMODE == 0:
        BDEBUG = False
    else:
        BDEBUG = True

    ROOT_PATH = config['PATH_SETTING']['ROOT_PATH']
    CPU_MAX = int(cpu_count() * float(config['PARAMS']['CPU_RATE']))
    RADARS = config['RADAR_SITES']['RADARS'].split(',')

    PATH_RR = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_RR']
    PATH_QPE = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_QPE']
    PATH_TMP = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_TMP']

    if not os.path.exists(PATH_QPE):
        try:
            os.makedirs(PATH_QPE)
        except:
            print('创建目录失败：' + PATH_QPE)
    if not os.path.exists(PATH_TMP):
        try:
            os.makedirs(PATH_TMP)
        except:
            print('创建目录失败：' + PATH_TMP)

    #降水产品格点设置
    GRID_XNUM = int(config['PARAMS']['GRID_XNUM'])#X方向格点数
    GRID_YNUM = int(config['PARAMS']['GRID_YNUM']) #Y方向格点数
    GRID_RESO = int(config['PARAMS']['GRID_RESO']) #网格分辨率，米

    ACC_HOURS = [int(mm) for mm in config['PARAMS']['acc_hours'].split(',')]
    data_save_hours = config['DATA_SAVE_SETTING']['DATA_SAVE_HOURS']


    # 输出的大网格设置
    MOSAIC_RESO = float(config['PARAMS']['MOSAIC_RESO']) #网格分辨率，度
    START_LAT = float(config['PARAMS']['SOUTH_LAT']) #南纬
    END_LAT = float(config['PARAMS']['NORTH_LAT']) #北纬
    START_LON = float(config['PARAMS']['WEST_LON']) #西经
    END_LON = float(config['PARAMS']['EAST_LON']) #东经

    with open('current_pid_s3_%d.txt'%RUNMODE,'wt') as f:
        f.write('current pid is: %s'%str(os.getpid()) + '  ,' + datetime.now().strftime('%Y-%m-%dT%H:%M:%S'))

    # 重复的时间间隔
    tstep = 10 # seconds

    if not BDEBUG:
        # 增加数据处理任务
        schedule.enter(0, 0, get_qpe_from_rr, (tstep,))    

        # 增加数据管理任务，定时删除旧文件
        schedule.enter(0, 0, _delete_old_data, (60,))

        schedule.run()
    else:
        get_qpe_from_rr(10)
    # get_qpe_from_rr(10,acc_hours,RADARS,PATH_RR,PATH_QPE,GRID_XNUM,GRID_YNUM,GRID_RESO)



# _*_ coding: utf-8 _*_

'''
专门用来解析色标文件的脚本
'''



import numpy as np
from matplotlib.colors import LinearSegmentedColormap, Normalize
import matplotlib.pyplot as plt
from matplotlib.colorbar import ColorbarBase
import re

# 请将色标文件中的solidcolor字段替换为color
def parse(file):
	lines = []
	vals = []
	red = []
	green = []
	blue = []
	last_color = None
	transit_flag = False
	with open(file, 'r',encoding='gb18030') as f:
		for line in f:
			if not line.lower().startswith('color:'):
				continue
			l = line.lower().lstrip('color: ').strip()
			lines.append(l)
	lines.sort(key=lambda x: float(x.split(' ')[0]))
	color_len = len(lines)
	for idx, l in enumerate(lines):
		segs = [i for i in l.split(' ') if i]
		vals.append(float(segs[0]))
		current_color = tuple(int(i) / 255 for i in segs[1:4])
		if color_len - idx == 2:
			transit_flag = transit_flag if transit_flag else False
		if not isinstance(last_color, tuple) and len(segs) == 3:
			red.append((0, current_color[0]))
			green.append((0, current_color[1]))
			blue.append((0, current_color[2]))
			last_color = current_color
		else:
			if len(segs) == 7 or color_len - idx == 2:
				transit_color = tuple(int(i) / 255 for i in segs[4:7])
				if transit_flag:
					red.append((last_color[0], current_color[0]))
					green.append((last_color[1], current_color[1]))
					blue.append((last_color[2], current_color[2]))
				else:
					red.append((current_color[0], current_color[0]))
					green.append((current_color[1], current_color[1]))
					blue.append((current_color[2], current_color[2]))
				last_color = transit_color
				transit_flag = True
			else:
				red.append((current_color[0], current_color[0]))
				green.append((current_color[1], current_color[1]))
				blue.append((current_color[2], current_color[2]))
				last_color = current_color
				transit_flag = False
	norm_array = (np.array(vals) - vals[0]) / (vals[-1] - vals[0])
	cdict = {'red':[], 'green':[], 'blue':[]}
	for idx in range(len(norm_array)):
		cdict['red'].append((norm_array[idx],) + red[idx])
		cdict['green'].append((norm_array[idx],) + green[idx])
		cdict['blue'].append((norm_array[idx],) + blue[idx])
	return LinearSegmentedColormap('cmap', cdict), Normalize(vals[0], vals[-1])

# 可以支持alpha，朱文剑，20210831
def parse_pro(file):
	lines = []
	vals = []
	red = []
	green = []
	blue = []
	alpha=[]
	last_color = None
	transit_flag = False
	units=''
	with open(file, 'r') as f:
		for line in f:
			if line.lower().startswith('units'):
				units = re.split(r"[ ]+",line)[1].strip()
			if not line.lower().startswith('color'):
				continue
			lines.append(line.strip())
	lines.sort(key=lambda x: float(re.split(r"[ ]+",x)[1]))
	color_len = len(lines)
	for idx, l in enumerate(lines):
		segs = [i for i in l.split(' ') if i]
		vals.append(float(segs[1]))
		
		current_color = tuple(int(i) / 255 for i in segs[2::])
		
		if color_len - idx == 2:
			transit_flag = transit_flag if transit_flag else False
		
		if len(segs) >=8 or color_len - idx == 2:
			if len(segs) == 10:
				transit_color = tuple(int(i) / 255 for i in segs[6:10])
			elif len(segs)==8:
				transit_color = tuple(int(i) / 255 for i in segs[5:8])
			
			if transit_flag:
				red.append((last_color[0], current_color[0]))
				green.append((last_color[1], current_color[1]))
				blue.append((last_color[2], current_color[2]))
				if segs[0]=='Color4:':
					alpha.append((last_color[3], current_color[3]))
				else:
					alpha.append((1, 1))
			else:
				red.append((current_color[0], current_color[0]))
				green.append((current_color[1], current_color[1]))
				blue.append((current_color[2], current_color[2]))
				if segs[0]=='Color4:':
					alpha.append((current_color[3], current_color[3]))
				else:
					alpha.append((1.0, 1.0))
			if len(segs) == 10 or len(segs) == 8:
				last_color = transit_color
				transit_flag = True
		else:
			red.append((current_color[0], current_color[0]))
			green.append((current_color[1], current_color[1]))
			blue.append((current_color[2], current_color[2]))
			if segs[0]=='Color4:':
				alpha.append((current_color[3], current_color[3]))
			else:
				alpha.append((1.0, 1.0))
			last_color = current_color
			transit_flag = False

	norm_array = (np.array(vals) - vals[0]) / (vals[-1] - vals[0])
	cdict = {'red':[], 'green':[], 'blue':[], 'alpha':[]}
	
	for idx in range(len(norm_array)):
		cdict['red'].append((norm_array[idx],) + red[idx])
		cdict['green'].append((norm_array[idx],) + green[idx])
		cdict['blue'].append((norm_array[idx],) + blue[idx])
		cdict['alpha'].append((norm_array[idx],) + alpha[idx])
	
	outdic=dict()
	outdic['cmap'] = LinearSegmentedColormap('cmap', cdict)
	outdic['norm'] = Normalize(vals[0], vals[-1])
	outdic['units'] = units
	return outdic

if __name__ == "__main__":

	outdic = parse_pro('gr2_colors/IR_dark_alpha.pal')
	fig = plt.figure(figsize=(3, 11))
	ax = plt.gca()
	cbar = ColorbarBase(ax, orientation="vertical", cmap=outdic['cmap'], norm=outdic['norm'])
	plt.show()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
多普勒雷达三维风场反演脚本
朱文剑

'''
# %%

import pyart
import pydda
import os
import matplotlib.pyplot as plt
import matplotlib
from metradar.util.get_tlogp_from_sharppy import get_profile
import numpy as np
import configparser
from datetime import datetime
import math
import argparse
import shutil
import sys
from metradar.io.decode_fmt_pyart import read_cnrad_fmt
import xarray as xr
# import warnings
# warnings.filterwarnings("ignore")


class GET_WIND_3D:
    # sub function for reading config file
    def ConfigFetchError(Exception):
        pass

    def _get_config_from_rcfile(self,rcfile):
        """
        Get configure information from config_dk_met_io.ini file.
        """
        # print(os.getcwd())
        rc = rcfile
        print(rc)
        try:
            config = configparser.ConfigParser()
            config.read(rc,encoding='UTF-8')
        except IOError as e:
            raise self.ConfigFetchError(str(e))
        except Exception as e:
            raise self.ConfigFetchError(str(e))

        return config

    def myprint(self,message):
        f = open('process_info.txt','at+',encoding='utf-8')
        f.write(message)
        f.write('\n')
        f.close()
        print(message)

    def write_to_progress_cross(self,value):
        f = open(self.picpath + os.sep + 'progress_cross.txt','wt')
        f.write(str(value))
        f.close()
        print('current progress = %d'%value + "%")

    def write_to_progress_main(self,value):
        f = open('progress_main.txt','wt')
        f.write(str(value))
        f.close()
        print('current progress = %d'%value + "%")

    def rad(self,d):
        return d * np.pi / 180.0

    def getDistance(self,lat1, lng1, lat2, lng2):
        # 计算两点间的距离，单位为千米
        EARTH_REDIUS = 6378.137
        radLat1 = self.rad(lat1)
        radLat2 = self.rad(lat2)
        a = radLat1 - radLat2
        b = self.rad(lng1) - self.rad(lng2)
        s = 2 * math.asin(math.sqrt(math.pow(math.sin(a/2), 2) + math.cos(radLat1) * math.cos(radLat2) * math.pow(math.sin(b/2), 2)))
        s = s * EARTH_REDIUS
        return s

    def write_to_gisinfo(self,):
        try:
            fout = open(self.picpath + os.sep + 'gisinfo.txt','wt',encoding='utf-8')
            fout.write('minLat=%.3f\n'%self.lat_min[0])
            fout.write('maxLat=%.3f\n'%self.lat_max[0])
            fout.write('minLon=%.3f\n'%self.lon_min[0])
            fout.write('maxLon=%.3f\n'%self.lon_max[0])
            self.myprint('minLat=%.3f'%self.lat_min[0])
            self.myprint('maxLat=%.3f'%self.lat_max[0])
            self.myprint('minLon=%.3f'%self.lon_min[0])
            self.myprint('maxLon=%.3f'%self.lon_max[0])
            fout.close()
            return True
        except:
            self.myprint(self.picpath + os.sep + 'gisinfo.txt' + ' open error!')
            return False

    def __init__(self,inifile='config.ini'):
        pass
        self.error=False
        if os.path.exists('process_info.txt'):
            os.remove('process_info.txt')

        if not os.path.exists(inifile):
            self.myprint(inifile + ' 不存在，请检查')
        config = self._get_config_from_rcfile(inifile)
        self.xlims = np.array(config['PARAM_SETTINGS']['XLIM'].split(', ')).astype(np.int64)
        self.ylims = np.array(config['PARAM_SETTINGS']['YLIM'].split(', ')).astype(np.int64)
        self.zlims = np.array(config['PARAM_SETTINGS']['ZLIM'].split(', ')).astype(int)
        self.time_files = config['PARAM_SETTINGS']['TIME_FILES']
        if len(self.time_files) !=14:
            self.myprint('时间参数设置错误，请重新设置 TIME_FILES 参数')
            self.error=True
            return None

        self.standard_lat = float(config['PARAM_SETTINGS']['CENTER_LAT'])
        self.standard_lon = float(config['PARAM_SETTINGS']['CENTER_LON'])

        self.ana_lat = float(config['PARAM_SETTINGS']['ANA_LAT'])
        self.ana_lon = float(config['PARAM_SETTINGS']['ANA_LON'])
        # bshowpic = int(config['PARAM_SETTINGS']['SHOW_PIC'])
        self.bsavepic = int(config['PARAM_SETTINGS']['SAVE_PIC'])
        if self.bsavepic == False:
            self.myprint('参数设置中设定不保存图片！')

        self.bshowpic = False
        if not self.bshowpic:
            matplotlib.use('Agg')
        else:
            matplotlib.use('MacOSX')

        self.radars = config['PARAM_SETTINGS']['RADAR_SITES'].split(', ')
        self.radarfilepath = config['PATH_SETTINGS']['RADAR_FILE_PATH']
        self.outpath = config['PATH_SETTINGS']['OUT_PATH']
        self.picpath = config['PATH_SETTINGS']['PIC_PATH']
        self.tlogp_filepath = config['PATH_SETTINGS']['TLOG_PATH']
        self.tlogp_filename = config['PATH_SETTINGS']['TLOG_FILE']

        if not os.path.exists(self.outpath):
            os.makedirs(self.outpath)

        if not os.path.exists(self.picpath):
            os.makedirs(self.picpath)

        self.xreso = int(config['PARAM_SETTINGS']['XRESO'])
        self.yreso = int(config['PARAM_SETTINGS']['YRESO'])
        self.zreso = int(config['PARAM_SETTINGS']['ZRESO'])

        self.lon_min,self.lat_min=pyart.core.cartesian_to_geographic_aeqd(self.xlims[0]*1000,self.ylims[0]*1000,self.standard_lon,self.standard_lat)
        self.lon_max,self.lat_max=pyart.core.cartesian_to_geographic_aeqd(self.xlims[1]*1000,self.ylims[1]*1000,self.standard_lon,self.standard_lat)

        if not self.write_to_gisinfo():
            self.write_to_progress_main(100)
            self.error=True
            self.myprint('write_to_gisinfo error!')
            return None

        self.g_xlim = (self.xlims[0]*1000,self.xlims[1]*1000)
        self.g_ylim = (self.ylims[0]*1000,self.ylims[1]*1000)
        self.g_zlim = (self.zlims[0]*1000,self.zlims[1]*1000)
        self.g_numx = int(len(range(self.xlims[0],self.xlims[1]))*1000/self.xreso+1) # x方向格点数
        self.g_numy = int(len(range(self.ylims[0],self.ylims[1]))*1000/self.yreso+1) # y方向格点数
        self.g_numz = int(len(range(self.zlims[0],self.zlims[1]))*1000/self.zreso+1) # z方向格点数

    def draw_all_pic(self,bdrawhori=True,):
        resultnames = []
        files = os.listdir(self.outpath)
        valid_files = []

        self.write_to_progress_cross(0)
        for ff in files:
            if ff.find('.nc')<=0:
                continue
            else:
                valid_files.append(ff)

        if len(valid_files) == 0:
            self.myprint('未找到nc格式的三维风场数据文件，请先调用--getwind方法')
            return False

        for ff in valid_files:
            resultnames.append(self.outpath + os.sep + ff)
            self.myprint(ff)
        
 
        s_x = self.getDistance(self.ana_lat,self.ana_lon,self.ana_lat,self.standard_lon)
        s_y = self.getDistance(self.ana_lat,self.ana_lon,self.standard_lat,self.ana_lon)
        self.myprint('拼图中心经度为：%.3f'%self.standard_lon)
        self.myprint('拼图中心纬度为：%.3f'%self.standard_lat)
        self.myprint('垂直剖面经度为：%.3f'%self.ana_lon)
        self.myprint('垂直剖面纬度为：%.3f'%self.ana_lat)
        self.myprint('X方向偏移量为：%d'%s_x)
        self.myprint('Y方向偏移量为：%d'%s_y)

        lev_yz = int((self.xlims[0] - s_x)/(self.xreso/1e3))
        lev_xz = int((s_y- self.ylims[0])/(self.yreso/1e3))
        self.myprint('X方向层次为：%d'%lev_xz)
        self.myprint('Y方向层次为：%d'%lev_yz)


        Grids = [xr.open_dataset(resultname) for resultname in resultnames]

        x=Grids[0].x.values
        y=Grids[0].y.values
        z=Grids[0].z.values

        uwind=Grids[0]['u'].values
        vwind=Grids[0]['v'].values
        wwind=Grids[0]['w'].values


        picpath_horiz_bar = self.picpath + os.sep + '水平风场'
        picpath_horiz_stream = self.picpath + os.sep + '水平流场'
        picpath_xz_bar = self.picpath + os.sep + 'XZ方向垂直风场'
        picpath_xz_stream = self.picpath + os.sep + 'XZ方向垂直流场'
        picpath_yz_bar = self.picpath + os.sep + 'YZ方向垂直风场'
        picpath_yz_stream = self.picpath + os.sep + 'YZ方向垂直流场'

        picpath_vor_stretch = self.picpath + os.sep + '拉伸涡度'
        picpath_vor_tilt = self.picpath + os.sep + '倾斜涡度'
 


        if not os.path.exists(picpath_horiz_bar):
            os.makedirs(picpath_horiz_bar)
        if not os.path.exists(picpath_horiz_stream):
            os.makedirs(picpath_horiz_stream)
        if not os.path.exists(picpath_xz_bar):
            os.makedirs(picpath_xz_bar)
        if not os.path.exists(picpath_xz_stream):
            os.makedirs(picpath_xz_stream)
        if not os.path.exists(picpath_yz_bar):
            os.makedirs(picpath_yz_bar)
        if not os.path.exists(picpath_yz_stream):
            os.makedirs(picpath_yz_stream)
        if not os.path.exists(picpath_vor_stretch):
            os.makedirs(picpath_vor_stretch)
        if not os.path.exists(picpath_vor_tilt):
            os.makedirs(picpath_vor_tilt)

        if bdrawhori:
            # draw barbs
            for ll in range(10): # self.g_numz
                fig, ax = plt.subplots(figsize=(8, 6))
                pydda.vis.plot_horiz_xsection_barbs(Grids, ax, 'reflectivity', level=ll,
                                                    # w_vel_contours=[3, 5,],
                                                    cmap='NWSRef', vmin=0, vmax=70,
                                                    barb_spacing_x_km=5,
                                                    barb_spacing_y_km=5,
                                                    show_lobes=False)
                ax.set_xlim(self.xlims.tolist())
                ax.set_ylim(self.ylims.tolist())
                ax.set_title(ax.get_title().replace('PyDDA','MultiRadar'))
                picfilename = 'horiz_xsection_barbs_%02d_%s.png'%(ll+1,self.time_files)
                if self.bsavepic:
                    fig.savefig(picpath_horiz_bar + os.sep + picfilename, 
                                format='png', transparent=False, dpi=300, pad_inches = 0)
                    self.myprint(picpath_horiz_bar + os.sep + picfilename + ' saved!')
                plt.close(fig)

            # draw streamline
            for ll in range(10): # self.g_numz
   
                fig, ax = plt.subplots(figsize=(8, 6))
                pydda.vis.plot_horiz_xsection_streamlines(Grids, ax, 'reflectivity', level=ll,
                                                        cmap='NWSRef', vmin=0, vmax=70,
                                                        #   w_vel_contours=[3, 5,],
                                                        show_lobes=False,
                                                        )
                ax.set_xlim(self.xlims.tolist())
                ax.set_ylim(self.ylims.tolist())
                ax.set_title(ax.get_title().replace('PyDDA','MultiRadar'))
                picfilename = 'horiz_xsection_streamlines_%02d_%s.png'%(ll+1,self.time_files)
                if self.bsavepic:
                    fig.savefig(picpath_horiz_stream + os.sep + picfilename, 
                                format='png', transparent=False, dpi=300, pad_inches = 0)
                    self.myprint(picpath_horiz_stream + os.sep + picfilename + ' saved!')

                plt.close(fig)
                
        try:
            #=================
            fig, ax = plt.subplots(figsize=(8, 4))
            pydda.vis.plot_xz_xsection_barbs(Grids, ax, 'reflectivity', level=lev_xz,
                                            #  w_vel_contours=[3, 5,],
                                            cmap='NWSRef', vmin=0, vmax=70,
                                            barb_spacing_x_km=2.0,
                                            barb_spacing_z_km=1.0)
            ax.set_xlim(self.xlims.tolist())
            ax.set_title(ax.get_title().replace('PyDDA','MultiRadar'))
            picfilename = 'xz_xsection_barbs_%06d_%06d_%s.png'%(self.ana_lat*1000,self.ana_lon*1000,self.time_files)
            
            if self.bsavepic:
                fig.savefig(picpath_xz_bar + os.sep + picfilename, 
                            format='png', transparent=False, dpi=300, pad_inches = 0)
                self.myprint(picpath_xz_bar + os.sep + picfilename + ' saved!')
            plt.close(fig)
            self.write_to_progress_cross(25)
        except:
            self.myprint('可能是垂直剖面的经纬度超出范围了，请重新设置再尝试！')
        #=================
        try:
            fig, ax = plt.subplots(figsize=(8, 4))
            pydda.vis.plot_xz_xsection_streamlines(Grids, None, 'reflectivity', level=lev_xz,
                                                cmap='NWSRef', vmin=0, vmax=70,
                                                #  w_vel_contours=[3, 5,],
                                                )
            ax.set_xlim(self.xlims.tolist())
            ax.set_title(ax.get_title().replace('PyDDA','MultiRadar'))
            picfilename = 'xz_xsection_streamlines_%06d_%06d_%s.png'%(self.ana_lat*1000,self.ana_lon*1000,self.time_files)
            if self.bsavepic:
                fig.savefig(picpath_xz_stream + os.sep + picfilename, 
                            format='png', transparent=False, dpi=300, pad_inches = 0)
                self.myprint(picpath_xz_stream + os.sep + picfilename + ' saved!')
            plt.close(fig)
            self.write_to_progress_cross(50)
        except:
            self.myprint('可能是垂直剖面的经纬度超出范围了，请重新设置再尝试！')
        #=================
        try:
            fig, ax = plt.subplots(figsize=(8, 4))
            pydda.vis.plot_yz_xsection_barbs(Grids, None, 'reflectivity', level=lev_yz,
                                            #  w_vel_contours=[ 3, 5, ],
                                            cmap='NWSRef', vmin=0, vmax=70,
                                            barb_spacing_y_km=2,
                                            barb_spacing_z_km=1)
            ax.set_xlim(self.ylims.tolist())
            ax.set_title(ax.get_title().replace('PyDDA','MultiRadar'))
            picfilename = 'yz_xsection_barbs_%06d_%06d_%s.png'%(self.ana_lat*1000,self.ana_lon*1000,self.time_files)
            if self.bsavepic:
                fig.savefig(picpath_yz_bar + os.sep + picfilename, 
                            format='png', transparent=False, dpi=300, pad_inches = 0)
                self.myprint(picpath_yz_bar + os.sep + picfilename + ' saved!')
            plt.close(fig)
            self.write_to_progress_cross(75)
        except:
            self.myprint('可能是垂直剖面的经纬度超出范围了，请重新设置再尝试！')
        #=================
        try:
            fig, ax = plt.subplots(figsize=(8, 4))
            pydda.vis.plot_yz_xsection_streamlines(Grids, None, 'reflectivity', level=lev_yz,
                                                cmap='NWSRef', vmin=0, vmax=70,
                                                #  w_vel_contours=[3, 5, ],
                                                )
            ax.set_xlim(self.ylims.tolist())
            ax.set_title(ax.get_title().replace('PyDDA','MultiRadar'))
            picfilename = 'yz_xsection_streamlines_%06d_%06d_%s.png'%(self.ana_lat*1000,self.ana_lon*1000,self.time_files)
            if self.bsavepic:
                fig.savefig(picpath_yz_stream + os.sep + picfilename, 
                            format='png', transparent=False, dpi=300, pad_inches = 0)
                self.myprint(picpath_yz_stream + os.sep + picfilename + ' saved!')
            plt.close(fig)
            self.write_to_progress_cross(100)
        except:
            self.myprint('可能是垂直剖面的经纬度超出范围了，请重新设置再尝试！')
        return True

    def do_wind_retrievel(self):

        self.write_to_progress_main(0)
        
        filepaths = []
        filenames = []

        for rd in self.radars:
            curpath = self.radarfilepath + os.sep + rd
            filepaths.append(curpath)
            files = os.listdir(curpath)
            files = sorted(files)

            files = [ff for ff in files if ff.find('Z_RADR')>=0]
            times = [datetime.strptime(ff[15:29], '%Y%m%d%H%M%S').timestamp()  for ff in files]


            if len(times) == 0:
                self.myprint('未找到 %s 雷达站的有效数据文件，请检查！'%rd)
                self.write_to_progress_main(100)
                return False

            ct = datetime.strptime(self.time_files, '%Y%m%d%H%M%S').timestamp()
            cidx = np.where(abs(np.array(times) - ct)==min(abs(np.array(times)-ct)))[0][0]
            if min(abs(np.array(times)-ct)) > 180:
                self.myprint('查找到 %s 雷达的资料中离规定时间最近的文件的观测时间与规定时间的差超过了180秒，请知悉！'%rd)
            self.myprint('找到的 %s 雷达的资料中与规定时间最接近的文件名为 %s '%(rd,files[cidx]))
            filenames.append(files[cidx])

        self.write_to_progress_main(5)    


        radar_objs=[]
        grid_objs=[]
        self.outname_grid=[]
        self.valid_radars=len(filenames)
        for nn in range(self.valid_radars):
            outname = filenames[nn].split('.')
            self.outname_grid.append(filenames[nn].replace(outname[-1],'wind.nc'))

        for nn in range(self.valid_radars):
            pass

            radar = read_cnrad_fmt(filepaths[nn] + os.sep + filenames[nn])
            # 退速速模糊
            newvel = pyart.correct.dealias_region_based(radar, vel_field='velocity')
            radar.add_field('corrected_velocity', newvel, replace_existing=True)
            radar_objs.append(radar)

            self.myprint('read %s over!'%filenames[nn])

            grid_objs.append(pyart.map.grid_from_radars(
            (radar_objs[nn],),
            grid_origin=[self.standard_lat,self.standard_lon],
            weighting_function = 'BARNES2',
            grid_shape=(self.g_numz, self.g_numy, self.g_numx),
            grid_limits=(self.g_zlim, self.g_ylim, self.g_xlim),
            fields=['reflectivity','velocity','corrected_velocity']).to_xarray())
            self.myprint('grid over!')

        self.write_to_progress_main(20)

        # step 2=============风场反演
        
        # 给grid对象添加部分属性point_x, point_y, point_z, AZ, EL, point_altitude
        # 按z，y，x的顺序，进行meshgrid
        point_z,point_y,point_x = np.mgrid[
            self.g_zlim[0]:self.g_zlim[1]+self.zreso:self.zreso,
            self.g_ylim[0]:self.g_ylim[1]+self.yreso:self.yreso,
            self.g_xlim[0]:self.g_xlim[1]+self.xreso:self.xreso
        ]

        # 将point_x, point_y, point_z添加到grid对象中
        for nn in range(len(grid_objs)):
            grid_objs[nn]['point_x'] = (('z','y','x'), point_x)
            grid_objs[nn]['point_y'] = (('z','y','x'), point_y)
            grid_objs[nn]['point_z'] = (('z','y','x'), point_z)

        # 根据z，y，x坐标，计算对应的方位角和仰角，并存储在AZ变量和EL变量中
        for nn in range(len(grid_objs)):
            grid_objs[nn]['AZ'] = (('z','y','x'), pyart.core.cartesian_to_antenna(
                grid_objs[nn]['point_x'].data,
                grid_objs[nn]['point_y'].data,
                grid_objs[nn]['point_z'].data)[1])
            grid_objs[nn]['EL'] = (('z','y','x'), pyart.core.cartesian_to_antenna(
                grid_objs[nn]['point_x'].data,
                grid_objs[nn]['point_y'].data,
                grid_objs[nn]['point_z'].data)[2])
        # 将高度信息存储在point_altitude变量中,直接采用z坐标值
        for nn in range(len(grid_objs)):
            grid_objs[nn]['point_altitude'] = (('z','y','x'), grid_objs[nn]['point_z'].data)
            
            
        # 添加探空数据
        if not os.path.exists(self.tlogp_filepath + os.sep + self.tlogp_filename):
            self.myprint(self.tlogp_filepath + os.sep + self.tlogp_filename + ' not exists! please check!')
            self.write_to_progress_main(100)
            return False

        profile = get_profile(self.tlogp_filepath,self.tlogp_filename)
        grid_objs[0]= pydda.initialization.make_wind_field_from_profile(
                grid_objs[0], profile, vel_field='corrected_velocity')
        
        # u_init, v_init, w_init = None,None,None
        self.myprint('探空数据 %s 加载完毕！'%self.tlogp_filename)
        # 速度退模糊暂时空缺，后期补上


        #进行多雷达三维风场反演
        # try:

        print('=================================')
        # return new_grid_list, parameters
        Grids, _ = pydda.retrieval.get_dd_wind_field(
                                                    grid_objs,
                                                    Co=100.0,
                                                    Cm=1500.0,
                                                    Cx=1e-2,
                                                    Cy=1e-2,
                                                    Cz=1e-2,
                                                    vel_name="corrected_velocity",
                                                    refl_field="reflectivity",
                                                    frz=5000.0,
                                                    engine="tensorflow", # 有GPU就用tensorflow，没有就用scipy或jax
                                                    mask_outside_opt=True,
                                                    upper_bc=1,
                                                )
        # except:
        #     self.myprint('get_dd_wind_field error!')
        #     self.write_to_progress_main(100)
        #     return False
        
        # 计算拉伸涡度和倾斜涡度
        Grids = self.calc_vortex(Grids)
        
        for nn in range(len(Grids)):
            # 将reflectivity小于10的格点设置为缺省值
            Grids[nn]['reflectivity'] = Grids[nn]['reflectivity'].where(Grids[nn]['reflectivity']>=10)
            outname = filenames[nn].split('.')
            
            Grids[nn].to_netcdf(self.outpath + os.sep + filenames[nn].replace(outname[-1],'wind.nc'))
            

        # exclude masked gates from the gridding
        # gatefilter = pyart.filters.GateFilter(Grids)
        # gatefilter.exclude_masked('reflectivity')

        self.myprint('write data to nc over!')
        self.write_to_progress_main(80)
        # print('All levels = %d'%g_numz)
        
        # self.draw_all_pic()

        self.write_to_progress_main(100)
        # if bshowpic:
        #     plt.show()

        if os.path.exists('process_info.txt'):
            shutil.copyfile('process_info.txt',self.picpath + os.sep + 'process_info.txt')

    #计算拉伸涡度和倾斜涡度
    def calc_vortex(self,Grids):
        self.myprint('starting calc_vortex......')
        

        
        if isinstance(Grids, list):
            uwind=Grids[0]['u'].values
            vwind=Grids[0]['v'].values
            wwind=Grids[0]['w'].values
        else:
            uwind=Grids['u'].values
            vwind=Grids['v'].values
            wwind=Grids['w'].values
        # 检查uwind的维度是否是四维
        if uwind.ndim != 4:
            raise ValueError("Expected uwind to be a 4D array with dimensions (time, z, y, x)")
        
        vor_stretch_data = np.zeros(uwind.shape,dtype=float)
        vor_tilt_data = np.zeros(uwind.shape,dtype=float)
        for tt in range(uwind.shape[0]):
            dudz,dudy,dudx = np.gradient(uwind[tt,:,:,:])
            dvdz,dvdy,dvdx = np.gradient(vwind[tt,:,:,:])
            dwdz,dwdy,dwdx = np.gradient(wwind[tt,:,:,:])
            
            for lev in range(uwind.shape[1]):
                # picname="stretch_%03d.png"%lev
                # print(lev)
                
                div=dudx[lev,:,:] + dvdy[lev,:,:]
                vor=dvdx[lev,:,:] - dudy[lev,:,:]
                
                u=uwind[tt,lev,:,:]
                v=vwind[tt,lev,:,:]
                #dw
                
                #拉伸项
                vor_stretch_data[tt,lev,:,:]=vor*div

                #倾斜项
                vor_tilt_data[tt,lev,:,:]=dwdy[lev,:,:]*dudz[lev,:,:]-dwdx[lev,:,:]*dvdz[lev,:,:]


        if isinstance(Grids, list):
            for nn in range(len(Grids)):
                Grids[nn]['vor_stretch'] = (('time','z','y','x'), vor_stretch_data)
                Grids[nn]['vor_tilt'] = (('time','z','y','x'), vor_tilt_data)
                
                # add attrs
                
                Grids[nn]['vor_stretch'].attrs['long_name'] = 'stretch vortex'
                Grids[nn]['vor_stretch'].attrs['units'] = 'm-2/s-2'
                Grids[nn]['vor_stretch'].attrs['coordinates'] = 't,z,y,x'
                
                Grids[nn]['vor_tilt'].attrs['long_name'] = 'tilting vortex'
                Grids[nn]['vor_tilt'].attrs['units'] = 'm-2/s-2'
                Grids[nn]['vor_tilt'].attrs['coordinates'] = 't,z,y,x'
            
        else:
            Grids['vor_stretch'] = (('time','z','y','x'), vor_stretch_data)
            Grids['vor_tilt'] = (('time','z','y','x'), vor_tilt_data)
            Grids['vor_stretch'].attrs['long_name'] = 'stretch vortex'
            Grids['vor_stretch'].attrs['units'] = 'm-2/s-2'
            Grids['vor_stretch'].attrs['coordinates'] = 't,z,y,x'
            
            Grids['vor_tilt'].attrs['long_name'] = 'tilting vortex'
            Grids['vor_tilt'].attrs['units'] = 'm-2/s-2'
            Grids['vor_tilt'].attrs['coordinates'] = 't,z,y,x'
        

        return Grids

            
    

if __name__ == "__main__":
    '''
    有两个函数可以调用：
    python main_pydda.py config_3dwind.ini --getwind
    python main_pydda.py config_3dwind.ini --drawcross
    其中第一个用来计算并绘图，第二个用来单独绘制任意点的垂直剖面
    '''
    with open('current_pid.txt','wt') as f:
        f.write('%s'%str(os.getpid()))
    def mainprog(inifile):

        _getwind = GET_WIND_3D(inifile) 
        if not _getwind.error:
            _getwind.do_wind_retrievel()
            _getwind.draw_all_pic()

        else:
            print('GET_WIND_3D Init error!')
            sys.exit(-1)
    
    def drawpic_cross(inifile):
        _getwind = GET_WIND_3D(inifile)
        if not _getwind.error:
            _getwind.draw_all_pic(bdrawhori=False)
        else:
            print('GET_WIND_3D Init error!')
            sys.exit(-1)

    parser = argparse.ArgumentParser(description='Do radar 3d wind retrievel.')
    parser.add_argument('inifile', metavar='inifile', type=str, 
                    help='ini filename for the mainprog')
    
    parser.add_argument('--getwind', dest='mainprog', action='store_const', 
                        const=mainprog,
                        help='do 3d wind retrievel')
    
    parser.add_argument('--drawcross', dest='drawpic_cross', action='store_const', 
                        const=drawpic_cross,
                        help='draw XZ and YZ pic')
    #是否为debug模式
    BDEBUG=True

    if BDEBUG:
        print('当前为debug模式，所以直接读取主目录下的config.ini, 请注意！')
        config_path = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/metradar/project/wind_retrieval/config_3dwind.ini'
        if not os.path.exists(config_path):
            print('%s not exists! please check!'%config_path)
            sys.exit(-1)
        mainprog(config_path)
        
    else:
        args = parser.parse_args()
        if not args.mainprog is None:
            args.mainprog(args.inifile)

        if not args.drawpic_cross is None:
            args.drawpic_cross(args.inifile)


# %%



# _*_ coding: utf-8 _*_

'''
定时扫描本地文件夹中的雷达基数据，并处理成单站qpe，用nc格式存储
'''

# %%
import os
from metradar.project.qpe.get_rainrate_func import get_rainrate
from datetime import datetime,timedelta
import sys
import configparser
import warnings
warnings.filterwarnings('ignore')
from multiprocessing import Pool,cpu_count,freeze_support
#添加时间控制器
import time
import sched
schedule = sched.scheduler(time.time, time.sleep)

BDEBUG=False
# sub function for reading config file
def ConfigFetchError(BaseException):
    pass

def _get_config_from_rcfile(rcfile):
    """
    Get configure information from config_dk_met_io.ini file.
    """
    # print(os.getcwd())
    print(rcfile)
    try:
        config = configparser.ConfigParser()
        config.read(rcfile,encoding='UTF-8')
    except IOError as e:
        raise ConfigFetchError(str(e))
    except Exception as e:
        raise ConfigFetchError(str(e))

    return config

def process_single(tstep=10):
    
    if not os.path.exists(PATH_RR):
        os.makedirs(PATH_RR)
    
    pools = Pool(CPU_MAX)
    print('共使用 %d 个逻辑CPU进行并行处理......'%CPU_MAX)
    for rd in RADARS:
       
       # 这里要改为递归查找文件，目录结构是ROOT_PATH ，年，月，日，站号
       
       # 递归处理所有子文件夹中的文件
        allfiles = []
        allpaths = []
        for root, dirs, files in os.walk(ROOT_PATH  + os.sep + BASEDATA_PATH):
            for file in files:
                if file.endswith('.bz2'):
                    if file.find(rd) < 0:
                        continue
                    allfiles.append(file)
                    allpaths.append(root)
                    
        curoutpath = PATH_RR + os.sep + rd
        if not os.path.exists(curoutpath):
            os.makedirs(curoutpath)

        allfiles = sorted(allfiles)

        for ff in allfiles:
            if ff.find('FMT.bin') < 0:
                allfiles.remove(ff)
                continue 
        
        bfiles_valid=0
        for filename in allfiles:
            #先判断该文件的qpe产品是否存在，如果已经存在就不重复处理，否则就处理
            outname = filename.replace('.bz2','_rr.nc')
            if not os.path.exists(curoutpath + os.sep + outname):
                bfiles_valid += 1
        if bfiles_valid > 0:
            print('%s 站，共找到 %d 个待处理的新数据，正在处理，请耐心等待......'%(rd,bfiles_valid))
        else:
            print('%s 站，共找到 %d 个待处理的新数据'%(rd,bfiles_valid))

        for nn in range(len(allfiles)):
            #先判断该文件的qpe产品是否存在，如果已经存在就不重复处理，否则就处理

            outname = allfiles[nn].replace('.bz2','_rr.nc')
            
            if os.path.exists(curoutpath + os.sep + outname):
                pass
                # print(curoutpath + os.sep + outname + ' 已存在，不重复处理！')
            else:        
                # print('正在处理 '+ filename)
                pools.apply_async(get_rainrate,(allpaths[nn],allfiles[nn],curoutpath,outname,GRID_XNUM,GRID_YNUM,GRID_RESO))
 
    pools.close()
    pools.join()
    if not BDEBUG:
        print('Waiting for new data......')
        schedule.enter(tstep, 0, process_single, (tstep,))


def _delete_old_data(tstep):
    
    print('delete expired data ...')
    curtime = datetime.now()
    pretime = curtime + timedelta(hours=-1 * int(data_save_hours))
    for rds in RADARS:
        #查询过期数据
        curpath = PATH_RR + os.sep + rds + os.sep
        if not os.path.exists(curpath):
            continue
        ff = os.listdir(curpath)
        for f in ff:
            t = os.path.getctime(curpath + os.sep + f)
           
            if t < pretime.timestamp():
                os.remove(curpath + os.sep + f)
                print(['Delete file:' + curpath + os.sep + f])
            
       
    schedule.enter(tstep, 0, _delete_old_data, (tstep,))


if __name__ == '__main__':
    freeze_support()
    print('Usage: python s2_pre_process_single_radar.py inifile')
    # 所有参数（包含脚本名）
    all_args = sys.argv  
    # 实际参数（排除脚本名）
    inifile = sys.argv[1:]
    

    #如果未指定配置文件，则选用默认的配置
    if len(inifile) ==0:
        inifile = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/metradar/project/qpe/main_qpe_cfg.ini'
    else:
        inifile = inifile[0]
    
    config = _get_config_from_rcfile(inifile)
    RUNMODE = int(config['COMMON']['RUN_MODE'])#X方向格点数
    if RUNMODE == 0:
        BDEBUG = False
    else:
        BDEBUG = True
    ROOT_PATH = config['PATH_SETTING']['ROOT_PATH']
    # BASEDATA_PATH =  ROOT_PATH + os.sep + config['PATH_SETTING']['BASEDATA_PATH']
    BASEDATA_PATH =  config['PATH_SETTING']['BASEDATA_PATH']

    RADARS = config['RADAR_SITES']['RADARS'].split(',')

    PATH_RR = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_RR']

    data_save_hours = config['DATA_SAVE_SETTING']['DATA_SAVE_HOURS']
    #降水产品格点设置
    GRID_XNUM = int(config['PARAMS']['GRID_XNUM'])#X方向格点数
    GRID_YNUM = int(config['PARAMS']['GRID_YNUM']) #Y方向格点数
    GRID_RESO = int(config['PARAMS']['GRID_RESO']) #网格分辨率，米
    CPU_MAX = int(cpu_count() * float(config['PARAMS']['CPU_RATE']))
    
    with open('current_pid_s2_%d.txt'%RUNMODE,'wt') as f:
        f.write('current pid is: %s'%str(os.getpid()) + '  ,' + datetime.now().strftime('%Y-%m-%dT%H:%M:%S'))

    # 重复的时间间隔
    tstep = 10 # seconds

    if not BDEBUG:
        # 增加数据处理任务
        schedule.enter(0, 0, process_single, (tstep,))    

        # 增加数据管理任务，定时删除旧文件
        schedule.enter(0, 0, _delete_old_data, (60,))


        schedule.run()
    else:
        process_single(tstep)
            


# %%




import numpy as np
def calc_td(temperature,relative_humidity):
    # temperature C degree
    # relative_humidity 0~1
    
    sat_pressure_0c = 6.112 #'millibar'

    # math:: 6.112 e^\frac{17.67T}{T + 243.5}
    vapor_pressure = relative_humidity * sat_pressure_0c * \
                    np.exp(17.67 * temperature/ (temperature +243.5))

    val = np.log(vapor_pressure / sat_pressure_0c)
    return (243.5 * val / (17.67 - val))

def wind_speed(u, v):
    r"""Compute the wind speed from u and v-components.

    Parameters
    ----------
    u : `pint.Quantity`
        Wind component in the X (East-West) direction
    v : `pint.Quantity`
        Wind component in the Y (North-South) direction

    Returns
    -------
    wind speed: `pint.Quantity`
        Speed of the wind

    See Also
    --------
    wind_components

    """
    speed = np.sqrt(u * u + v * v)
    return speed

def wind_direction(u, v, convention='from'):
    

    wdir = 90 - np.rad2deg(np.arctan2(-v, -u))
    origshape = wdir.shape
    wdir = np.atleast_1d(wdir)

    # Handle oceanographic convection
    if convention == 'to':
        wdir -= 180
    elif convention not in ('to', 'from'):
        raise ValueError('Invalid kwarg for "convention". Valid options are "from" or "to".')

    mask = wdir <= 0
    if np.any(mask):
        wdir[mask] += 360
    # avoid unintended modification of `pint.Quantity` by direct use of magnitude
    calm_mask = (np.asarray(u) == 0.) & (np.asarray(v) == 0.)
    # np.any check required for legacy numpy which treats 0-d False boolean index as zero
    if np.any(calm_mask):
        wdir[calm_mask] = 0
    return wdir.reshape(origshape)

def geopotential_to_height(geopotential):
    r"""Compute height above sea level from a given geopotential.

    Calculates the height above mean sea level from geopotential using the following formula,
    which is derived from the definition of geopotential as given in [Hobbs2006]_ Pg. 69 Eq
    3.21, along with an approximation for variation of gravity with altitude:

    .. math:: z = \frac{\Phi R_e}{gR_e - \Phi}

    (where :math:`\Phi` is geopotential, :math:`z` is height, :math:`R_e` is average Earth
    radius, and :math:`g` is standard gravity).


    Parameters
    ----------
    geopotential : `pint.Quantity`
        Geopotential

    Returns
    -------
    `pint.Quantity`
        Corresponding value(s) of height above sea level

    """
    Re = 6371008.7714
    g = 9.80665
    return (geopotential * Re) / (g * Re - geopotential)

if __name__ == "__main__":
    pass

    g = 9.80665
    print(geopotential_to_height(5884*g))


'''
从sharppy格式的探空数据中获取风廓线，作为背景场

朱文剑
'''

# %%
import os
import re
import numpy as np
from pyart.core.wind_profile import HorizontalWindProfile
import warnings
warnings.filterwarnings("ignore")

def get_profile(filepath,filename):
    """
    该函数读取sharppy格式的探空文件，提取高度、风速、风向等信息，构建并返回一个HorizontalWindProfile对象，便于后续风廓线分析和可视化。
    Parameters
    ----------
    filepath : str
        文件路径
    filename : str
        文件名
    Returns
    -------
    outprofile : HorizontalWindProfile
    
    补充说明：
    
    """

    fin = open(filepath + os.sep + filename,'rt')

    nflag_title=0
    nflag_raw=0
    nflag=0
    profile=dict()
    defaultvalue = '99999'
    for line in fin.readlines():
        # print(line)
        nflag = nflag + 1
        if line.find('%TITLE%') >=0 :
            nflag_title = nflag
        if line.find('%RAW%') >=0 :
            nflag_raw = nflag

            # 遇到文档结束
        if line.find('%END%') >=0 :
            break
        #获取站号，时间信息
        if nflag == nflag_title + 1:
            profile['staname'] = line.split(' ')[0]
        
        #获取经纬度信息
        if nflag == nflag_title + 2:
            tmp = re.findall(r"\d+\.?\d*",line)
            if len(tmp) == 2:
                profile['lon'] = tmp[0]
                profile['lat'] = tmp[1]
            else:
                profile['lon'] = defaultvalue
                profile['lat'] = defaultvalue
        
        #获取变量名称
        if nflag == nflag_title + 3:
            tmp = line.split(' ')
            varnames = [i.strip('\n') for i in tmp if i != '']
            # print(varnames)
            if len(varnames) ==6:
                for dd in range(len(varnames)):
                    profile[varnames[dd]]=[]
                    # exec('{} = []'.format(varnames[dd]))
            else:
                print('变量个数不够，请检查')
                break

        # 数据
        if nflag > nflag_raw:
            tmp = re.findall(r"\d+\.?\d*",line)
            if len(tmp) ==6:
                values = np.fromstring( line, sep=',')
                value_str = [str(value) for value in values]
                for dd in range(len(value_str)):
                    # exec('{}.append({})'.format(varnames[dd], value_str[dd]))
                    profile[varnames[dd]].append(values[dd])
            else:
                continue

    outprofile = HorizontalWindProfile(
            profile['HGHT'], np.array(profile['WSPD'])/1.94, profile['WDIR'], latitude=np.tile(float(profile['lat']), len(profile['HGHT'])), longitude=np.tile(float(profile['lon']), len(profile['HGHT'])))

    fin.close()
    print('profile read over!')
    return outprofile   

if __name__ == '__main__':
    filepath = 'tlogp'
    filename = '22060608.59280'
    profile = get_profile(filepath,filename)
    pass



# %%



# _*_ coding: utf-8 _*_


# 多进程绘图，提高历史模式下绘图效率

# %%绘图
import os
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.transforms import offset_copy
from matplotlib.font_manager import FontProperties
import cartopy.crs as ccrs
import nmc_met_graphics.plot.mapview as nmc_draw
from netCDF4 import Dataset
import warnings

warnings.filterwarnings('ignore')

import matplotlib as mpl
from datetime import datetime,timedelta
from multiprocessing import Pool,Manager,freeze_support,cpu_count
import sys

from metradar.config import CONFIG

# 资源文件路径
RESOURCES_PATH = CONFIG.get('SETTING','RESOURCES_PATH')
FONT_FILE = RESOURCES_PATH + '/fonts/YaHeiConsolasHybrid_1.12.ttf'

BDEBUG = False

manager = Manager()
# 创建一个dict，装入每个qpe拼图的文件的名称和修改时间
shared_dict = manager.dict()

if not BDEBUG:
    # linux : export MPLBACKEND=Agg
    mpl.use('Agg')

import configparser
#添加时间控制器
import time
import sched
schedule = sched.scheduler(time.time, time.sleep)
# sub function for reading config file
def ConfigFetchError(Exception):
    pass

def _get_config_from_rcfile(rcfile):
    """
    Get configure information from config_dk_met_io.ini file.
    """

    print(rcfile)
    try:
        config = configparser.ConfigParser()
        config.read(rcfile,encoding='UTF-8')
    except IOError as e:
        raise ConfigFetchError(str(e))
    except Exception as e:
        raise ConfigFetchError(str(e))

    return config

def get_datetime_from_filename1(filename):
    timstr = filename[11:23]
    fyear = int(timstr[0:4])
    fmonth = int(timstr[4:6])
    fday = int(timstr[6:8])
    fhour = int(timstr[8:10])
    fmin = int(timstr[10:12])
    fsec = 0
    ft = datetime(fyear,fmonth,fday,fhour,fmin,fsec).timestamp()
    return ft

def draw_single(params: dict):
    filename = params['filename']
    pic_path = params['pic_path']
    mosaic_path = params['mosaic_path']
    acc_hours = params['acc_hours']
    print('process file: ' + filename + '  acc_hours: ' + str(acc_hours))
    if filename.find('%02dhr.nc'%acc_hours) < 0 or filename.find('.lock') > 0:
        return -2
    outname = filename.replace('.nc','.png')

    if not BDEBUG:
        pass
        # 只要超过20分钟就不重新绘制
        if (datetime.utcnow().timestamp() - get_datetime_from_filename1(filename)) > 20*60:
            return -1

        if os.path.exists(pic_path + os.sep + outname) and (datetime.utcnow().timestamp() - get_datetime_from_filename1(filename)) > 20*60 :
            # print(pic_path + os.sep + outname + ' 已存在，且过时20分钟，不重新绘制！')
            return -3

    if not os.path.exists(mosaic_path + os.sep + filename):
        print(mosaic_path + os.sep + filename + ' 文件不存在！')
        return -4

    try:
        mosaic_file = Dataset(mosaic_path + os.sep + filename,'r')
    except:
        print(mosaic_path + os.sep + filename + ' 文件读取失败！')
        return -5
    
    # 检测临时状态
    curt_file = datetime.fromtimestamp(os.path.getmtime(mosaic_path + os.sep + filename))
    if filename in list(shared_dict.keys()):
        if curt_file == shared_dict[filename]:
            # 修改时间无变化，那么就不画图
            print(filename + ' 无修改，不重复绘图！')
            return 1

    shared_dict[filename] = curt_file   
    mosaic_grid = mosaic_file.variables['qpe_mosaic'][:,:] * mosaic_file.variables['qpe_mosaic'].unit_scale
    used_hours = mosaic_file.used_hours
    used_sites = mosaic_file.used_sites
    used_lons = mosaic_file.used_lons
    used_lats = mosaic_file.used_lats
    unused_sites = mosaic_file.unused_sites
    unused_lons = mosaic_file.unused_lons
    unused_lats = mosaic_file.unused_lats

    # 
    #降水产品格点设置
    START_LAT= mosaic_file.start_lat
    END_LAT  = mosaic_file.end_lat
    START_LON= mosaic_file.start_lon
    END_LON  = mosaic_file.end_lon
    GRID_RESO = mosaic_file.reso #网格分辨率，经纬度，度

    mosaic_lats = np.arange(START_LAT,END_LAT+GRID_RESO/2,GRID_RESO)
    mosaic_lons = np.arange(START_LON,END_LON+GRID_RESO/2,GRID_RESO)
    
    m = nmc_draw.BaseMap(projection='PlateCarree',central_longitude=110.0,res='h')
    fig =plt.figure(figsize=(8,8))
    ax = plt.axes(projection=m.proj)
    ax.set_extent([START_LON, END_LON, START_LAT, END_LAT])
    mosaic_grid[mosaic_grid<0.5] = np.nan
    projection = ccrs.PlateCarree()

    # cmap = parse_cmap('pyart_NWSRef')
    colors_1 = [[183,245,167],[120,215,110],[65,185,60],[50,140,45],[100,185,255],[0,5,255],[255,0,255]]
    colors_2 = [[168,242,142],[65,185,60],[100,185,255],[0,5,255],[255,0,255],[128,0,64]]
    levs_1 = [0,2.5,5,10,25,50,100,999]
    levs_2 = [0,10,25,50,100,250,999]
    if acc_hours < 3:
        colors = np.array(colors_1)/255
        levs = levs_1
    else:
        colors = np.array(colors_2)/255
        levs = levs_2
    cmap = mpl.colors.ListedColormap(colors, 'precipitation')
    pm = m.contourf(mosaic_lons, mosaic_lats, mosaic_grid[:,:],ax=ax,colors =colors,levels=levs)
    pm.set_clim(0,999)
    nmc_draw.add_china_map_2cartopy(ax, name='county', edgecolor='darkgray', lw=0.5)
    nmc_draw.add_china_map_2cartopy(ax, name='province', edgecolor='k', lw=1)
    
    tt1 = datetime.strptime(mosaic_file.product_time, '%Y%m%d%H%M') + timedelta(hours=8)
    tt2 = datetime.strptime(mosaic_file.last_update, '%Y%m%d%H%M') + timedelta(hours=8)
    m.title(left_title = '%d hours Radar QPE(%s)'%(acc_hours,tt1.strftime('%Y%m%d%H%M')), ax=ax, font_size=13)
    m.title(right_title = 'Last update:%s'%tt2.strftime('%Y%m%d%H%M'), ax=ax, font_size=13)
    m.gridlines()
    # m.cities()
    if np.array(used_sites).size > 0:
        ax.plot(used_lons, used_lats, marker='o', color='red', markersize=5, linewidth=0,
                alpha=0.7, transform=ccrs.PlateCarree())
        font = FontProperties(size=8, weight='bold')
        geodetic_transform = ccrs.Geodetic()._as_mpl_transform(ax)
        text_transform = offset_copy(geodetic_transform, units='dots', x=0,y=5)
        if np.array(used_hours).size == 1:
            ax.text(used_lons, used_lats, str(used_hours) + 'hr',verticalalignment='bottom', horizontalalignment='center',
                transform=text_transform, fontproperties=font, color='black')
        else:
            for bbx in range(np.array(used_hours).size):
                ax.text(used_lons[bbx], used_lats[bbx], str(used_hours[bbx]) + 'hr',verticalalignment='bottom', horizontalalignment='center',
                    transform=text_transform, fontproperties=font, color='black')


    if np.array(unused_sites).size > 0:
        ax.plot(unused_lons, unused_lats, marker='o', color='black', markersize=6, linewidth=0,
                alpha=0.7, transform=ccrs.PlateCarree())
    m.colorbar(pm)
    plt.savefig( pic_path + os.sep + outname,dpi=150)
    # 防止内存泄露
    plt.close()
    print(pic_path + os.sep + outname + ' done!')

    return 0
    
def drawpic(tstep=10):

    for acc_hours in ACC_HOURS:
        pic_path = pic_path_ori + os.sep + '%02dhr'%acc_hours
        if not os.path.exists(pic_path):
            os.makedirs(pic_path)
        mosaic_path = mosaic_path_ori + os.sep + '%02dhr'%acc_hours
        if not os.path.exists(mosaic_path):
            print(mosaic_path + ' 路径不存在！')
            continue
        files = os.listdir(mosaic_path)
        files = sorted(files)
        if len(files) == 0:
            continue
        all_pms =[]
        for filename in files:
            params = {}
            params['filename'] = filename
            params['pic_path'] = pic_path
            params['mosaic_path'] = mosaic_path
            params['acc_hours'] = acc_hours
            all_pms.append(params)

        # 构建多进程pool
        freeze_support()
        cores = CPU_MAX
        if cores > len(files):
            cores = len(files)
        print('总共采用 %d 个核心，处理 %d 个文件'%(cores,len(files)))
        pool = Pool(processes = cores)
        pool.map(draw_single, all_pms) 
        pool.close()
        pool.join()

    if not BDEBUG: # 实时模式
        print('Waiting for new data......')
        schedule.enter(tstep, 0, drawpic, (tstep,))

def _delete_old_data(tstep):
    
    print('delete expired data ...')
    curtime = datetime.now()
    pretime = curtime + timedelta(hours=-1*int(data_save_hours))
    
    #查询过期数据
    
    for acc_hours in ACC_HOURS:          
        curpath = pic_path_ori + os.sep + '%02dhr'%acc_hours
        if not os.path.exists(curpath):
            return False
        ff = os.listdir(curpath)
        for f in ff:
            t = os.path.getctime(curpath + os.sep + f)
            if t < pretime.timestamp():
                os.remove(curpath + os.sep + f)
                print(['Delete file:' + curpath + os.sep + f])

    schedule.enter(tstep, 0, _delete_old_data, (tstep,))

if __name__ == '__main__':
    
    print('Usage: python s5_draw_qpe_mosaic.py inifile')
    # 所有参数（包含脚本名）
    all_args = sys.argv  
    # 实际参数（排除脚本名）
    inifile = sys.argv[1:]
    

    #如果未指定配置文件，则选用默认的配置
    if len(inifile) ==0:
        inifile = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/metradar/project/qpe/main_qpe_cfg.ini'
        # inifile = 'archive_main_qpe_cfg.ini'
    else:
        inifile = inifile[0]
    
    config = _get_config_from_rcfile(inifile)
    RUNMODE = int(config['COMMON']['RUN_MODE'])#X方向格点数
    
    if RUNMODE == 0:
        BDEBUG = False
    else:
        BDEBUG = True
 
    ROOT_PATH = config['PATH_SETTING']['ROOT_PATH']
    ACC_HOURS = [int(mm) for mm in config['PARAMS']['acc_hours'].split(',')]
    mosaic_path_ori = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_QPE_MOSAIC']
    pic_path_ori = ROOT_PATH + os.sep + config['GRAPH']['PIC_PATH']
    DPI = int(config['GRAPH']['PIC_DPI'])
    data_save_hours = config['DATA_SAVE_SETTING']['DATA_SAVE_HOURS']
    CPU_MAX = int(cpu_count() * float(config['PARAMS']['CPU_RATE']))

    
    with open('current_pid_s5.txt','wt') as f:
        f.write('current pid is: %s'%str(os.getpid()) + '  ,' + datetime.now().strftime('%Y-%m-%dT%H:%M:%S'))

    import matplotlib as mpl
    import shutil

    # 获取 Matplotlib 字体目录
    font_dir = mpl.get_data_path() + '/fonts/ttf/'
    # 复制字体文件（需管理员权限）
    shutil.copy(FONT_FILE, font_dir)
    cache_dir = mpl.get_cachedir()
    shutil.rmtree(cache_dir) 

    if not BDEBUG:
        # 重复的时间间隔
        tstep = 10 # seconds

        # 增加数据处理任务
        schedule.enter(0, 0, drawpic, (tstep,))    

        # 增加数据管理任务，定时删除旧文件
        schedule.enter(0, 0, _delete_old_data, (60,))

        schedule.run()
    else:
        drawpic(10)